# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'lession_editor_GUI_slim_orez.ui'
#
# Created: Tue Sep 29 10:52:55 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(721, 839)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setFocusPolicy(QtCore.Qt.ClickFocus)
        MainWindow.setTabShape(QtGui.QTabWidget.Rounded)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.splitter = QtGui.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName(_fromUtf8("splitter"))
        self.frame_R = QtGui.QFrame(self.splitter)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_R.sizePolicy().hasHeightForWidth())
        self.frame_R.setSizePolicy(sizePolicy)
        self.frame_R.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame_R.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_R.setObjectName(_fromUtf8("frame_R"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.frame_R)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.verticalLayout_7 = QtGui.QVBoxLayout()
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.figure_L_CB = QtGui.QComboBox(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.figure_L_CB.sizePolicy().hasHeightForWidth())
        self.figure_L_CB.setSizePolicy(sizePolicy)
        self.figure_L_CB.setObjectName(_fromUtf8("figure_L_CB"))
        self.verticalLayout_7.addWidget(self.figure_L_CB)
        self.left_view_btns_F = QtGui.QFrame(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.left_view_btns_F.sizePolicy().hasHeightForWidth())
        self.left_view_btns_F.setSizePolicy(sizePolicy)
        self.left_view_btns_F.setMinimumSize(QtCore.QSize(0, 20))
        self.left_view_btns_F.setFrameShape(QtGui.QFrame.StyledPanel)
        self.left_view_btns_F.setFrameShadow(QtGui.QFrame.Raised)
        self.left_view_btns_F.setObjectName(_fromUtf8("left_view_btns_F"))
        self.horizontalLayout_12 = QtGui.QHBoxLayout(self.left_view_btns_F)
        self.horizontalLayout_12.setObjectName(_fromUtf8("horizontalLayout_12"))
        self.view_L_BTN = QtGui.QPushButton(self.left_view_btns_F)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.view_L_BTN.sizePolicy().hasHeightForWidth())
        self.view_L_BTN.setSizePolicy(sizePolicy)
        self.view_L_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.view_L_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.view_L_BTN.setText(_fromUtf8(""))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8("icons/Eye.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.view_L_BTN.setIcon(icon)
        self.view_L_BTN.setIconSize(QtCore.QSize(24, 24))
        self.view_L_BTN.setObjectName(_fromUtf8("view_L_BTN"))
        self.horizontalLayout_12.addWidget(self.view_L_BTN)
        self.show_im_L_BTN = QtGui.QPushButton(self.left_view_btns_F)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_im_L_BTN.sizePolicy().hasHeightForWidth())
        self.show_im_L_BTN.setSizePolicy(sizePolicy)
        self.show_im_L_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.show_im_L_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.show_im_L_BTN.setToolTip(_fromUtf8("Show data"))
        self.show_im_L_BTN.setText(_fromUtf8(""))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8("icons/Stock graph.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.show_im_L_BTN.setIcon(icon1)
        self.show_im_L_BTN.setIconSize(QtCore.QSize(24, 24))
        self.show_im_L_BTN.setObjectName(_fromUtf8("show_im_L_BTN"))
        self.horizontalLayout_12.addWidget(self.show_im_L_BTN)
        self.show_labels_L_BTN = QtGui.QPushButton(self.left_view_btns_F)
        self.show_labels_L_BTN.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_labels_L_BTN.sizePolicy().hasHeightForWidth())
        self.show_labels_L_BTN.setSizePolicy(sizePolicy)
        self.show_labels_L_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.show_labels_L_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.show_labels_L_BTN.setText(_fromUtf8(""))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8("icons/Blue tag.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.show_labels_L_BTN.setIcon(icon2)
        self.show_labels_L_BTN.setIconSize(QtCore.QSize(24, 24))
        self.show_labels_L_BTN.setObjectName(_fromUtf8("show_labels_L_BTN"))
        self.horizontalLayout_12.addWidget(self.show_labels_L_BTN)
        self.show_contours_L_BTN = QtGui.QPushButton(self.left_view_btns_F)
        self.show_contours_L_BTN.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_contours_L_BTN.sizePolicy().hasHeightForWidth())
        self.show_contours_L_BTN.setSizePolicy(sizePolicy)
        self.show_contours_L_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.show_contours_L_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.show_contours_L_BTN.setText(_fromUtf8(""))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8("icons/Brush.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.show_contours_L_BTN.setIcon(icon3)
        self.show_contours_L_BTN.setIconSize(QtCore.QSize(24, 24))
        self.show_contours_L_BTN.setObjectName(_fromUtf8("show_contours_L_BTN"))
        self.horizontalLayout_12.addWidget(self.show_contours_L_BTN)
        self.verticalLayout_7.addWidget(self.left_view_btns_F)
        self.horizontalLayout.addLayout(self.verticalLayout_7)
        self.line_4 = QtGui.QFrame(self.frame_R)
        self.line_4.setFrameShape(QtGui.QFrame.VLine)
        self.line_4.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_4.setObjectName(_fromUtf8("line_4"))
        self.horizontalLayout.addWidget(self.line_4)
        self.verticalLayout_8 = QtGui.QVBoxLayout()
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.figure_R_CB = QtGui.QComboBox(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.figure_R_CB.sizePolicy().hasHeightForWidth())
        self.figure_R_CB.setSizePolicy(sizePolicy)
        self.figure_R_CB.setObjectName(_fromUtf8("figure_R_CB"))
        self.verticalLayout_8.addWidget(self.figure_R_CB)
        self.right_view_btns_F = QtGui.QFrame(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.right_view_btns_F.sizePolicy().hasHeightForWidth())
        self.right_view_btns_F.setSizePolicy(sizePolicy)
        self.right_view_btns_F.setMinimumSize(QtCore.QSize(0, 20))
        self.right_view_btns_F.setFrameShape(QtGui.QFrame.StyledPanel)
        self.right_view_btns_F.setFrameShadow(QtGui.QFrame.Raised)
        self.right_view_btns_F.setObjectName(_fromUtf8("right_view_btns_F"))
        self.horizontalLayout_13 = QtGui.QHBoxLayout(self.right_view_btns_F)
        self.horizontalLayout_13.setObjectName(_fromUtf8("horizontalLayout_13"))
        self.view_R_BTN = QtGui.QPushButton(self.right_view_btns_F)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.view_R_BTN.sizePolicy().hasHeightForWidth())
        self.view_R_BTN.setSizePolicy(sizePolicy)
        self.view_R_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.view_R_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.view_R_BTN.setText(_fromUtf8(""))
        self.view_R_BTN.setIcon(icon)
        self.view_R_BTN.setIconSize(QtCore.QSize(24, 24))
        self.view_R_BTN.setObjectName(_fromUtf8("view_R_BTN"))
        self.horizontalLayout_13.addWidget(self.view_R_BTN)
        self.show_im_R_BTN = QtGui.QPushButton(self.right_view_btns_F)
        self.show_im_R_BTN.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_im_R_BTN.sizePolicy().hasHeightForWidth())
        self.show_im_R_BTN.setSizePolicy(sizePolicy)
        self.show_im_R_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.show_im_R_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.show_im_R_BTN.setText(_fromUtf8(""))
        self.show_im_R_BTN.setIcon(icon1)
        self.show_im_R_BTN.setIconSize(QtCore.QSize(24, 24))
        self.show_im_R_BTN.setObjectName(_fromUtf8("show_im_R_BTN"))
        self.horizontalLayout_13.addWidget(self.show_im_R_BTN)
        self.show_labels_R_BTN = QtGui.QPushButton(self.right_view_btns_F)
        self.show_labels_R_BTN.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_labels_R_BTN.sizePolicy().hasHeightForWidth())
        self.show_labels_R_BTN.setSizePolicy(sizePolicy)
        self.show_labels_R_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.show_labels_R_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.show_labels_R_BTN.setText(_fromUtf8(""))
        self.show_labels_R_BTN.setIcon(icon2)
        self.show_labels_R_BTN.setIconSize(QtCore.QSize(24, 24))
        self.show_labels_R_BTN.setObjectName(_fromUtf8("show_labels_R_BTN"))
        self.horizontalLayout_13.addWidget(self.show_labels_R_BTN)
        self.show_contours_R_BTN = QtGui.QPushButton(self.right_view_btns_F)
        self.show_contours_R_BTN.setEnabled(False)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_contours_R_BTN.sizePolicy().hasHeightForWidth())
        self.show_contours_R_BTN.setSizePolicy(sizePolicy)
        self.show_contours_R_BTN.setMinimumSize(QtCore.QSize(40, 0))
        self.show_contours_R_BTN.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.show_contours_R_BTN.setText(_fromUtf8(""))
        self.show_contours_R_BTN.setIcon(icon3)
        self.show_contours_R_BTN.setIconSize(QtCore.QSize(24, 24))
        self.show_contours_R_BTN.setObjectName(_fromUtf8("show_contours_R_BTN"))
        self.horizontalLayout_13.addWidget(self.show_contours_R_BTN)
        self.verticalLayout_8.addWidget(self.right_view_btns_F)
        self.horizontalLayout.addLayout(self.verticalLayout_8)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.viewer_F = QtGui.QFrame(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.viewer_F.sizePolicy().hasHeightForWidth())
        self.viewer_F.setSizePolicy(sizePolicy)
        self.viewer_F.setMinimumSize(QtCore.QSize(0, 0))
        self.viewer_F.setFrameShape(QtGui.QFrame.Box)
        self.viewer_F.setObjectName(_fromUtf8("viewer_F"))
        self.verticalLayout_2.addWidget(self.viewer_F)
        self.frame_7 = QtGui.QFrame(self.frame_R)
        self.frame_7.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame_7.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_7.setObjectName(_fromUtf8("frame_7"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.frame_7)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.slice_number_L_LBL = QtGui.QLabel(self.frame_7)
        self.slice_number_L_LBL.setMaximumSize(QtCore.QSize(60, 16777215))
        self.slice_number_L_LBL.setObjectName(_fromUtf8("slice_number_L_LBL"))
        self.horizontalLayout_3.addWidget(self.slice_number_L_LBL)
        self.slice_L_SB = QtGui.QScrollBar(self.frame_7)
        self.slice_L_SB.setOrientation(QtCore.Qt.Horizontal)
        self.slice_L_SB.setObjectName(_fromUtf8("slice_L_SB"))
        self.horizontalLayout_3.addWidget(self.slice_L_SB)
        self.slice_R_SB = QtGui.QScrollBar(self.frame_7)
        self.slice_R_SB.setEnabled(True)
        self.slice_R_SB.setOrientation(QtCore.Qt.Horizontal)
        self.slice_R_SB.setObjectName(_fromUtf8("slice_R_SB"))
        self.horizontalLayout_3.addWidget(self.slice_R_SB)
        self.slice_number_R_LBL = QtGui.QLabel(self.frame_7)
        self.slice_number_R_LBL.setMaximumSize(QtCore.QSize(60, 16777215))
        self.slice_number_R_LBL.setObjectName(_fromUtf8("slice_number_R_LBL"))
        self.horizontalLayout_3.addWidget(self.slice_number_R_LBL)
        self.verticalLayout_2.addWidget(self.frame_7)
        self.horizontalLayout_11 = QtGui.QHBoxLayout()
        self.horizontalLayout_11.setObjectName(_fromUtf8("horizontalLayout_11"))
        self.slice_number_C_LBL = QtGui.QLabel(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.slice_number_C_LBL.sizePolicy().hasHeightForWidth())
        self.slice_number_C_LBL.setSizePolicy(sizePolicy)
        self.slice_number_C_LBL.setMinimumSize(QtCore.QSize(100, 0))
        self.slice_number_C_LBL.setObjectName(_fromUtf8("slice_number_C_LBL"))
        self.horizontalLayout_11.addWidget(self.slice_number_C_LBL)
        self.slice_C_SB = QtGui.QScrollBar(self.frame_R)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.slice_C_SB.sizePolicy().hasHeightForWidth())
        self.slice_C_SB.setSizePolicy(sizePolicy)
        self.slice_C_SB.setOrientation(QtCore.Qt.Horizontal)
        self.slice_C_SB.setObjectName(_fromUtf8("slice_C_SB"))
        self.horizontalLayout_11.addWidget(self.slice_C_SB)
        self.verticalLayout_2.addLayout(self.horizontalLayout_11)
        self.verticalLayout_4.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 721, 25))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        self.menuSerie = QtGui.QMenu(self.menubar)
        self.menuSerie.setObjectName(_fromUtf8("menuSerie"))
        self.menuAction = QtGui.QMenu(self.menubar)
        self.menuAction.setObjectName(_fromUtf8("menuAction"))
        self.menuShow = QtGui.QMenu(self.menubar)
        self.menuShow.setObjectName(_fromUtf8("menuShow"))
        self.menuTools = QtGui.QMenu(self.menubar)
        self.menuTools.setObjectName(_fromUtf8("menuTools"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.action_load_serie_1 = QtGui.QAction(MainWindow)
        self.action_load_serie_1.setObjectName(_fromUtf8("action_load_serie_1"))
        self.action_load_serie_2 = QtGui.QAction(MainWindow)
        self.action_load_serie_2.setObjectName(_fromUtf8("action_load_serie_2"))
        self.action_run = QtGui.QAction(MainWindow)
        self.action_run.setObjectName(_fromUtf8("action_run"))
        self.action_delete = QtGui.QAction(MainWindow)
        self.action_delete.setObjectName(_fromUtf8("action_delete"))
        self.action_restart = QtGui.QAction(MainWindow)
        self.action_restart.setObjectName(_fromUtf8("action_restart"))
        self.action_show_color_model = QtGui.QAction(MainWindow)
        self.action_show_color_model.setObjectName(_fromUtf8("action_show_color_model"))
        self.action_show_object_list = QtGui.QAction(MainWindow)
        self.action_show_object_list.setObjectName(_fromUtf8("action_show_object_list"))
        self.action_circle = QtGui.QAction(MainWindow)
        self.action_circle.setObjectName(_fromUtf8("action_circle"))
        self.actionRuler = QtGui.QAction(MainWindow)
        self.actionRuler.setObjectName(_fromUtf8("actionRuler"))
        self.actionAxial = QtGui.QAction(MainWindow)
        self.actionAxial.setObjectName(_fromUtf8("actionAxial"))
        self.actionFrontal = QtGui.QAction(MainWindow)
        self.actionFrontal.setObjectName(_fromUtf8("actionFrontal"))
        self.actionSagital = QtGui.QAction(MainWindow)
        self.actionSagital.setObjectName(_fromUtf8("actionSagital"))
        self.action_calculate_color_model = QtGui.QAction(MainWindow)
        self.action_calculate_color_model.setObjectName(_fromUtf8("action_calculate_color_model"))
        self.menuSerie.addAction(self.action_load_serie_1)
        self.menuSerie.addAction(self.action_load_serie_2)
        self.menuAction.addAction(self.action_calculate_color_model)
        self.menuAction.addSeparator()
        self.menuAction.addAction(self.action_run)
        self.menuAction.addAction(self.action_delete)
        self.menuAction.addAction(self.action_restart)
        self.menuShow.addAction(self.action_show_color_model)
        self.menuShow.addAction(self.action_show_object_list)
        self.menuTools.addAction(self.action_circle)
        self.menubar.addAction(self.menuSerie.menuAction())
        self.menubar.addAction(self.menuAction.menuAction())
        self.menubar.addAction(self.menuShow.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow", None))
        self.view_L_BTN.setToolTip(_translate("MainWindow", "Show figure", None))
        self.show_labels_L_BTN.setToolTip(_translate("MainWindow", "Show labels", None))
        self.show_contours_L_BTN.setToolTip(_translate("MainWindow", "Show contours", None))
        self.view_R_BTN.setToolTip(_translate("MainWindow", "Show figure", None))
        self.show_im_R_BTN.setToolTip(_translate("MainWindow", "Show data", None))
        self.show_labels_R_BTN.setToolTip(_translate("MainWindow", "Show labels", None))
        self.show_contours_R_BTN.setToolTip(_translate("MainWindow", "Show contours", None))
        self.slice_number_L_LBL.setText(_translate("MainWindow", "0/0", None))
        self.slice_number_R_LBL.setText(_translate("MainWindow", "0/0", None))
        self.slice_number_C_LBL.setText(_translate("MainWindow", "slice # = 0/0", None))
        self.menuSerie.setTitle(_translate("MainWindow", "Data", None))
        self.menuAction.setTitle(_translate("MainWindow", "Action", None))
        self.menuShow.setTitle(_translate("MainWindow", "Show", None))
        self.menuTools.setTitle(_translate("MainWindow", "Tools", None))
        self.action_load_serie_1.setText(_translate("MainWindow", "Load serie #1...", None))
        self.action_load_serie_2.setText(_translate("MainWindow", "Load serie #2...", None))
        self.action_run.setText(_translate("MainWindow", "Run localization", None))
        self.action_delete.setText(_translate("MainWindow", "Delete localization", None))
        self.action_restart.setText(_translate("MainWindow", "Restart localization", None))
        self.action_show_color_model.setText(_translate("MainWindow", "Color Model", None))
        self.action_show_object_list.setText(_translate("MainWindow", "Object List", None))
        self.action_circle.setText(_translate("MainWindow", "Circle", None))
        self.actionRuler.setText(_translate("MainWindow", "Ruler", None))
        self.actionAxial.setText(_translate("MainWindow", "Axial", None))
        self.actionFrontal.setText(_translate("MainWindow", "Frontal", None))
        self.actionSagital.setText(_translate("MainWindow", "Sagital", None))
        self.action_calculate_color_model.setText(_translate("MainWindow", "Calculate color model", None))




#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import unittest
import sys
# sys.path.append("./src/")

import scipy.io

import logging
logger = logging.getLogger(__name__)

import argparse
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
from matplotlib.widgets import Slider, Button, RadioButtons

class py3DSeedEditor:
    """ Viewer and seed editor for 2D and 3D data. 

    py3DSeedEditor(img, ...)

    img: 2D or 3D grayscale data
    voxelsizemm: size of voxel, default is [1, 1, 1]
    initslice: 0
    colorbar: True/False, default is True
    cmap: colormap
    zaxis: axis with slice numbers


    ed = py3DSeedEditor(img)
    ed.show()
    selected_seeds = ed.seeds

    """

    def __init__(self, img, voxelsizemm=[1,1,1], initslice = 0 , colorbar = True,
            cmap = matplotlib.cm.Greys_r, seeds = None, contour = None, zaxis=0,
            mouse_button_map= {1:1, 2:2, 3:3, 4:4, 5:5, 6:6, 7:7, 8:8},
            windowW = [], windowC = [],
            range_per_slice = False
            ):
        self.fig = plt.figure()

        if len(img.shape) == 2:
            imgtmp = img
            img = np.zeros([1, imgtmp.shape[0], imgtmp.shape[1]])
            img[-1,:,:] = imgtmp
            zaxis = 0

        # Rotate data in depndecy on zaxispyplot
        img = self._rotate_start(img, zaxis)
        seeds = self._rotate_start(seeds, zaxis)
        contour = self._rotate_start(contour, zaxis)

        self.rotated_back = False
        self.zaxis = zaxis

        # if True, intensity range is calculated per slice = better visualisation for
        # higher number of labels
        self.range_per_slice = range_per_slice

        #self.ax = self.fig.add_subplot(111)
        self.imgshape = list(img.shape)
        self.img = img
        self.actual_slice = initslice
        self.colorbar = colorbar
        self.cmap = cmap 
        if seeds == None:
            self.seeds = np.zeros(self.imgshape, np.int8)
        else:
            self.seeds = seeds
        if not (windowW and windowC):
            self.imgmax = np.max(img)
            self.imgmin = np.min(img)
        else:
            self.imgmax = windowC + (windowW / 2)
            self.imgmin = windowC - (windowW / 2)

        """ Mapping mouse button to class number. Default is normal order"""
        self.button_map = mouse_button_map

        self.contour = contour

        self.press = None
        self.press2 = None

# language
        self.texts = {'btn_delete':'Delete', 'btn_close': 'Close'}

        #iself.fig.subplots_adjust(left=0.25, bottom=0.25)
        self.ax = self.fig.add_axes([0.2, 0.3, 0.7,0.6])

        self.draw_slice()

        if self.colorbar:
            self.fig.colorbar(self.imsh)

        # user interface look
        axcolor = 'lightgoldenrodyellow'
        ax_actual_slice = self.fig.add_axes([0.2, 0.2, 0.6, 0.03], axisbg=axcolor)
        self.actual_slice_slider = Slider(ax_actual_slice, 'Slice', 0, 
                self.imgshape[2], valinit=initslice)
        
        # conenction to wheel events
        self.fig.canvas.mpl_connect('scroll_event', self.on_scroll)
        self.actual_slice_slider.on_changed(self.sliceslider_update)
# draw
        self.fig.canvas.mpl_connect('button_press_event', self.on_press)
        self.fig.canvas.mpl_connect('button_release_event', self.on_release)
        self.fig.canvas.mpl_connect('motion_notify_event', self.on_motion)


# delete seeds
        self.ax_delete_seeds = self.fig.add_axes([0.2,0.1,0.1,0.075])
        self.btn_delete = Button(self.ax_delete_seeds, self.texts['btn_delete'])
        self.btn_delete.on_clicked(self.callback_delete)

# close button
        self.ax_delete_seeds = self.fig.add_axes([0.7,0.1,0.1,0.075])
        self.btn_delete = Button(self.ax_delete_seeds, self.texts['btn_close'])
        self.btn_delete.on_clicked(self.callback_close)

        self.draw_slice()


    def _rotate_start(self, data, zaxis):
        if data != None:
            if zaxis == 0:
                data = np.transpose(data,(1,2,0))
            elif zaxis == 2:
                pass
            else:
                print "problem with zaxis in _rotate_start()"

        return data


    def _rotate_end(self, data, zaxis):
        if data != None:
            if self.rotated_back == False:
                if zaxis == 0:
                    data = np.transpose(data,(2,0,1))
                elif zaxis == 2:
                    pass
                else:
                    print "problem with zaxis in _rotate_start()"
            else:
                print "There is a danger in calling show() twice"

        return data


    def update_slice(self):
        #TODO tohle je tu kvuli contour, neumim ji odstranit jinak
        self.ax.cla()
        self.draw_slice()


    def draw_slice(self):
        self.actual_slice = np.int(self.actual_slice)
        sliceimg = self.img[:, :, self.actual_slice]
        if self.range_per_slice:
            self.imsh = self.ax.imshow(sliceimg, self.cmap, vmin=sliceimg.min(), vmax=sliceimg.max(), interpolation='nearest')
        else:
            self.imsh = self.ax.imshow(sliceimg, self.cmap, vmin=self.imgmin, vmax=self.imgmax, interpolation='nearest')
        self.ax.imshow(self.prepare_overlay(self.seeds[:, :, self.actual_slice]), interpolation='nearest', vmin=self.imgmin, vmax=self.imgmax)
        
        if self.contour != None:
            try:
                # exception catch problem with none object in image
                ctr = self.ax.contour(self.contour[:, :, self.actual_slice], 1, linewidths=2)
            except:
                pass

        self.fig.canvas.draw()


    def next_slice(self):
        self.actual_slice = self.actual_slice + 1
        if self.actual_slice >= self.imgshape[2]:
            self.actual_slice = 0


    def prev_slice(self):
        self.actual_slice = self.actual_slice - 1
        if self.actual_slice < 0:
            self.actual_slice = self.imgshape[2] - 1


    def sliceslider_update(self, val):
# zaokrouhlení
        #self.actual_slice_slider.set_val(round(self.actual_slice_slider.val))
        self.actual_slice = round(val)
        self.update_slice()


    def prepare_overlay(self,seeds):
        sh = list(seeds.shape)
        if len(sh) == 2:
            sh.append(4)
        else:
            sh[2] = 4
        overlay = np.zeros(sh)

        overlay[:,:,0] = (seeds == 1)
        overlay[:,:,1] = (seeds == 2)
        overlay[:,:,2] = (seeds == 3)
        overlay[:,:,3] = (seeds > 0)

        return overlay


    def show(self):
        """ Function run viewer window.
        """
        plt.show()
        # Rotate data in depndecy on zaxis
        self.img = self._rotate_end(self.img, self.zaxis)
        self.seeds = self._rotate_end(self.seeds, self.zaxis)
        self.contour = self._rotate_end(self.contour, self.zaxis)
        self.rotated_back = True
        return self.seeds


    def on_scroll(self, event):
        ''' mouse wheel is used for setting slider value'''
        if event.button == 'up':
            self.next_slice()
        if event.button == 'down':
            self.prev_slice()
        self.actual_slice_slider.set_val (self.actual_slice)


## malování -------------------
    def on_press(self, event):
        'on but-ton press we will see if the mouse is over us and store some data'
        if event.inaxes != self.ax: return
        #contains, attrd = self.rect.contains(event)
        #if not contains: return
        #print 'event contains', self.rect.xy
        #x0, y0 = self.rect.xy
        self.press = [event.xdata], [event.ydata], event.button
        #self.press1 = True


    def on_motion(self, event):
        'on motion we will move the rect if the mouse is over us'
        if self.press is None: return

        if event.inaxes != self.ax: return
        #print event.inaxes

        x0, y0, btn = self.press
        x0.append(event.xdata)
        y0.append(event.ydata)


    def on_release(self, event):
        'on release we reset the press data'
        if self.press is None: return
        #print self.press
        x0, y0, btn = self.press
        if btn == 1:
            color = 'r'
        elif btn == 2:
            color = 'b'

        #button Mapping
        btn = self.button_map[btn]

        self.set_seeds(y0, x0, self.actual_slice, btn )

        self.press = None
        self.update_slice()


    def callback_delete(self, event):
        self.seeds[:,:,self.actual_slice] = 0
        self.update_slice()


    def callback_close(self, event):
        matplotlib.pyplot.clf()
        matplotlib.pyplot.close()


    def set_seeds(self, px, py, pz, value = 1, voxelsizemm = [1,1,1], cursorsizemm = [1,1,1]):
        assert len(px) == len(py) , 'px and py describes a point, their size must be same'

        for i, item in enumerate(px):
            self.seeds[item, py[i], pz] = value


    def get_seed_sub(self, label):
        """ Return list of all seeds with specific label
        """
        sx, sy, sz = np.nonzero(self.seeds == label)

        return sx, sy, sz


    def get_seed_val(self, label):
        """ Return data values for specific seed label"""
        return self.img[self.seeds==label]

        
def generate_data(shp=[16,20,24]):
    """ Generating data """

    x = np.ones(shp)
# inserting box
    x[4:-4, 6:-2, 1:-6] = -1
    x_noisy = x + np.random.normal(0, 0.6, size=x.shape)
    return x_noisy


# --------------------------main------------------------------
if __name__ == "__main__":
    # input parser
    parser = argparse.ArgumentParser(description='Segment vessels from liver. For example call py3DSeedEditor -f lena')
    parser.add_argument('-f','--filename',  
            #default = '../jatra/main/step.mat',
            default = 'lena',
            help='*.mat file with variables "data", "segmentation" and "threshod"')
    parser.add_argument('-d', '--debug', action='store_true',
            help='run in debug mode')
    parser.add_argument('-e3', '--example3d', action='store_true',
            help='run with 3D example data')
    parser.add_argument('-t', '--tests', action='store_true', 
            help='run unittest')
    parser.add_argument('-o', '--outputfile', type=str,
        default='output.mat',help='output file name')
    args = parser.parse_args()


    if args.debug:
        logger.setLevel(logging.DEBUG)

    if args.tests:
        # hack for use argparse and unittest in one module
        sys.argv[1:]=[]
        unittest.main()

    if args.example3d:
        data = generate_data()
    elif args.filename == 'lena':
        from scipy import misc
        data = misc.lena()
    else:
    #   load all 
        mat = scipy.io.loadmat(args.filename)
        logger.debug( mat.keys())

        # load specific variable
        dataraw = scipy.io.loadmat(args.filename, variable_names=['data'])
        data = dataraw['data']

    pyed = py3DSeedEditor(data)
    output = pyed.show()

    scipy.io.savemat(args.outputfile,{'data':output})
    pyed.get_seed_val(1)




__author__ = 'tomas'

import sys
from PyQt4 import QtGui, QtCore
import numpy as np
import My_table_model as mtm

from qrangeslider import QRangeSlider

from objects_widget_GUI import Ui_Form

class Objects_widget(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)

        # area range slider
        self.area_RS = QRangeSlider()
        self.area_RS.setMin(0)
        self.area_RS.setMax(10000)
        self.area_RS.setRange(10, self.area_RS.max())
        self.ui.area_layout.addWidget(self.area_RS)

        # self.area_RS.endValueChanged.connect(self.max_area_changed_callback)
        # self.area_RS.startValueChanged.connect(self.min_area_changed_callback)

        # density range slider
        self.density_RS = QRangeSlider()
        self.density_RS.setMin(-50)
        self.density_RS.setMax(1000)
        self.density_RS.setRange(10, self.density_RS.max())
        self.ui.density_layout.addWidget(self.density_RS)

    # def max_area_changed_callback(self, value):
    #     print 'max area changed to ', value
    #
    # def min_area_changed(self, value):
    #     print 'min area changed to ', value

    def set_area_range(self, areas):
        self.area_RS.setMin(min(areas))
        self.area_RS.setMax(max(areas))
        self.area_RS.setRange()

    def set_density_range(self, densities):
        self.density_RS.setMin(min(densities))
        self.density_RS.setMax(max(densities))
        self.density_RS.setRange()

    def keyPressEvent(self, QKeyEvent):
        print 'hist widget key event: ',
        if QKeyEvent.key() == QtCore.Qt.Key_Escape:
            print 'Escape'
            if self.ui.objects_TV.selectedIndexes():
                self.ui.objects_TV.clearSelection()
            else:
                self.close()

    # self.cc.objects_filtration(self.selected_objects_labels, min_area=self.ui.min_area_SL.value(), max_area=value)
    #     # self.fill_table(self.cc.labels[self.cc.filtered_idxs], self.cc.areas[self.cc.filtered_idxs], self.cc.comps[self.cc.filtered_idxs])
    #     self.fill_table(self.cc.actual_data.lesions, self.cc.actual_data.labels, self.cc.filtered_idxs)
    #     # TODO: nasleduje prasarna
    #     if self.view_L.show_mode == self.view_L.SHOW_LABELS:
    #         self.show_labels_L_callback()
    #     if self.view_R.show_mode == self.view_R.SHOW_LABELS:
    #         self.show_labels_R_callback()

if __name__ == '__main__':

    from objects_widget_GUI import Ui_Form

    import Lesion
    app = QtGui.QApplication(sys.argv)

    labels = np.array([[1, 1, 0, 2, 0],
                       [1, 1, 0, 2, 0],
                       [0, 0, 0, 2, 0],
                       [3, 0, 4, 0, 5],
                       [3, 0, 4, 0, 0]], dtype=np.int)
    labels = np.dstack((labels, labels, labels))
    lesions = Lesion.extract_lesions(labels, data=labels)

    objects_w = Objects_widget()
    table_model = mtm.MyTableModel(lesions, labels)
    objects_w.ui.objects_TV.setModel(table_model)
    objects_w.show()

    sys.exit(app.exec_())


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'area_hist_widget_GUI.ui'
#
# Created: Thu Aug 27 04:01:06 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName(_fromUtf8("Frame"))
        Frame.resize(400, 209)
        Frame.setFrameShape(QtGui.QFrame.StyledPanel)
        Frame.setFrameShadow(QtGui.QFrame.Raised)
        self.verticalLayout = QtGui.QVBoxLayout(Frame)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.hist_F = QtGui.QFrame(Frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hist_F.sizePolicy().hasHeightForWidth())
        self.hist_F.setSizePolicy(sizePolicy)
        self.hist_F.setFrameShape(QtGui.QFrame.StyledPanel)
        self.hist_F.setFrameShadow(QtGui.QFrame.Raised)
        self.hist_F.setObjectName(_fromUtf8("hist_F"))
        self.verticalLayout.addWidget(self.hist_F)
        self.frame_2 = QtGui.QFrame(Frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setFrameShape(QtGui.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_2.setObjectName(_fromUtf8("frame_2"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.frame_2)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.mean_LBL = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mean_LBL.sizePolicy().hasHeightForWidth())
        self.mean_LBL.setSizePolicy(sizePolicy)
        self.mean_LBL.setObjectName(_fromUtf8("mean_LBL"))
        self.horizontalLayout.addWidget(self.mean_LBL)
        self.label_3 = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout.addWidget(self.label_3)
        self.std_LBL = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.std_LBL.sizePolicy().hasHeightForWidth())
        self.std_LBL.setSizePolicy(sizePolicy)
        self.std_LBL.setObjectName(_fromUtf8("std_LBL"))
        self.horizontalLayout.addWidget(self.std_LBL)
        self.verticalLayout.addWidget(self.frame_2)

        self.retranslateUi(Frame)
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        Frame.setWindowTitle(_translate("Frame", "Frame", None))
        self.label.setText(_translate("Frame", "mean = ", None))
        self.mean_LBL.setText(_translate("Frame", "0", None))
        self.label_3.setText(_translate("Frame", "std = ", None))
        self.std_LBL.setText(_translate("Frame", "0", None))




__author__ = 'tomas'


import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage.morphology as scindimor
import scipy.ndimage.measurements as scindimea
import scipy.ndimage.interpolation as scindiint
import scipy.ndimage as scindi
import scipy.stats as scista

import skimage.segmentation as skiseg
import skimage.morphology as skimor
import skimage.filter as skifil
import skimage.exposure as skiexp
import skimage.measure as skimea
import skimage.transform as skitra

import cv2
import pygco

import tools
import py3DSeedEditor
# from mayavi import mlab

import TumorVisualiser

from sklearn import metrics
from sklearn.cluster import KMeans

import pickle

import Viewer_3D

import cv2

import Data
import Lesion

import logging
logger = logging.getLogger(__name__)
logging.basicConfig()

class Computational_core():

    def __init__(self, fname, params, status_bar):
        # self.params = self.init_params()
        # self.params = params
        # self.models = None
        self.status_bar = status_bar

        # self.labels = None  # list of unique labels
        # self.filtered_idxs = None

        # ext_list = ('pklz', 'pickle')
        # self.fname = fname
        # self.data_1 = Data.Data()
        # self.data_2 = Data.Data()
        # self.actual_data = self.data_1
        # self.active_serie = 1

        # self.objects = list()  # list of segmented lesions

        # loading data - both series if provided
        # if len(self.fname) > 0:
        #     name = self.fname[0]
        #     if name.split('.')[-1] in ext_list:
        #         self.data_1.load_data(name)
        #
        #     else:
        #         msg = 'Wrong data type, supported extensions: ', ', '.join(ext_list)
        #         raise IOError(msg)
        # if len(self.fname) > 1:
        #     name = self.fname[1]
        #     if name.split('.')[-1] in ext_list:
        #         self.data_2.load_data(name)
        #     else:
        #         msg = 'Wrong data type, supported extensions: ', ', '.join(ext_list)
        #         raise IOError(msg)

        # smooth data if allowed
        # self.data_1.data = self.smooth_data(self.data_1.data)
        # self.data_2.data = self.smooth_data(self.data_2.data)

    def data_zoom(self, data, voxelsize_mm, working_voxelsize_mm):
        zoom = voxelsize_mm / (1.0 * working_voxelsize_mm)
        data_res = scindi.zoom(data, zoom, mode='nearest', order=1).astype(np.int16)
        return data_res

    def zoom_to_shape(self, data, shape):
        zoom = np.array(shape, dtype=np.float) / np.array(data.shape, dtype=np.float)
        data_res = scindi.zoom(data, zoom, mode='nearest', order=1).astype(np.int16)
        return data_res

    # def load_pickle_data(self, fname, slice_idx=-1):
    #     fcontent = None
    #     try:
    #         import gzip
    #         f = gzip.open(fname, 'rb')
    #         fcontent = f.read()
    #         f.close()
    #     except Exception as e:
    #         logger.warning("Input gzip exception: " + str(e))
    #         f = open(fname, 'rb')
    #         fcontent = f.read()
    #         f.close()
    #     data_dict = pickle.loads(fcontent)
    #
    #     data = tools.windowing(data_dict['data3d'], level=self.params['win_level'], width=self.params['win_width'])
    #
    #     mask = data_dict['segmentation']
    #
    #     voxel_size = data_dict['voxelsize_mm']
    #     # data = data_zoom(data, voxel_size, params['working_voxelsize_mm'])
    #     # mask = data_zoom(data_dict['segmentation'], voxel_size, params['working_voxelsize_mm'])
    #
    #
    #     if slice_idx != -1:
    #         data = data[slice_idx, :, :]
    #         mask = mask[slice_idx, :, :]
    #
    #     return data, mask, voxel_size

    def estimate_healthy_pdf(self, data, mask, params):
        perc = params['perc']
        k_std_l = params['k_std_h']
        simple_estim = params['healthy_simple_estim']
        show_me = params['show_healthy_pdf_estim']

        # data = tools.smoothing_tv(data.astype(np.uint8), tv_weight)
        ints = data[np.nonzero(mask)]
        hist, bins = skiexp.histogram(ints, nbins=256)
        if simple_estim:
            mu, sigma = scista.norm.fit(ints)
        else:
            ints = data[np.nonzero(mask)]

            n_pts = mask.sum()
            perc_in = n_pts * perc / 100

            peak_idx = np.argmax(hist)
            n_in = hist[peak_idx]
            win_width = 0

            while n_in < perc_in:
                win_width += 1
                n_in = hist[peak_idx - win_width:peak_idx + win_width].sum()

            idx_start = bins[peak_idx - win_width]
            idx_end = bins[peak_idx + win_width]
            inners_m = np.logical_and(ints > idx_start, ints < idx_end)
            inners = ints[np.nonzero(inners_m)]

            # liver pdf -------------
            mu = bins[peak_idx] + params['hack_healthy_mu']
            sigma = k_std_l * np.std(inners) + params['hack_healthy_sigma']

        mu = int(mu)
        sigma = int(sigma)
        rv = scista.norm(mu, sigma)

        if show_me:
            plt.figure()
            plt.subplot(211)
            plt.plot(bins, hist)
            plt.title('histogram with max peak')
            plt.hold(True)
            plt.plot([mu, mu], [0, hist.max()], 'g')
            ax = plt.axis()
            plt.axis([0, 256, ax[2], ax[3]])
            # plt.subplot(212), plt.plot(bins, rv_l.pdf(bins), 'g')
            x = np.arange(0, 256, 0.1)
            plt.subplot(212), plt.plot(x, rv.pdf(x), 'g')
            plt.hold(True)
            plt.plot(mu, rv.pdf(mu), 'go')
            ax = plt.axis()
            plt.axis([0, 256, ax[2], ax[3]])
            plt.title('estimated normal pdf of healthy parenchym')
            # plt.show()

        return rv

    def estimate_outlier_pdf(self, data, mask, rv_healthy, outlier_type, params):
        prob_w = params['prob_w']
        show_me = params['show_outlier_pdf_estim']

        if outlier_type == 'hypo':
            hack_mu = params['hack_hypo_mu']
            hack_sigma = params['hack_hypo_sigma']
        elif outlier_type == 'hyper':
            hack_mu = params['hack_hyper_mu']
            hack_sigma = params['hack_hyper_sigma']

        probs = rv_healthy.pdf(data) * mask
        # hist, bins = skiexp.histogram(probs, nbins=100)
        # plt.figure()
        # plt.plot(bins, hist)
        # plt.show()

        max_prob = rv_healthy.pdf(rv_healthy.mean())
        # mean_prob = np.mean(probs[np.nonzero(mask)])
        print 'max_prob = %.3f' % max_prob
        # print 'mean prob = %.3f' % mean_prob
        prob_t = prob_w * max_prob

        ints_out_m = probs < prob_t * mask

        # TumorVisualiser.run(data, ints_out_m, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)

        ints_out = data[np.nonzero(ints_out_m)]
        hist, bins = skiexp.histogram(ints_out, nbins=256)

        if outlier_type == 'hypo':
            ints = ints_out[np.nonzero(ints_out < rv_healthy.mean())]
            # m = ints_out_m * (data < rv_healthy.mean())
            # TumorVisualiser.run(data, m, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)
        elif outlier_type == 'hyper':
            ints = ints_out[np.nonzero(ints_out > rv_healthy.mean())]
            # m = ints_out_m * (data > rv_healthy.mean())
            # TumorVisualiser.run(data, m, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)
        else:
            print 'Wrong outlier specification.'
            return

        mu, sigma = scista.norm.fit(ints)

        # hack for moving pdfs of hypo and especially of hyper furhter from the pdf of healthy parenchyma
        mu += hack_mu
        sigma += hack_sigma
        #-----------

        mu = int(mu)
        sigma = int(sigma)
        rv = scista.norm(mu, sigma)

        if show_me:
            plt.figure()
            plt.subplot(211)
            plt.plot(bins, hist)
            plt.title('histogram with max peak')
            plt.hold(True)
            plt.plot([mu, mu], [0, hist.max()], 'g')
            ax = plt.axis()
            plt.axis([0, 256, ax[2], ax[3]])
            # plt.subplot(212), plt.plot(bins, rv_l.pdf(bins), 'g')
            x = np.arange(0, 256, 0.1)
            plt.subplot(212), plt.plot(x, rv.pdf(x), 'g')
            plt.hold(True)
            plt.plot(mu, rv.pdf(mu), 'go')
            ax = plt.axis()
            plt.axis([0, 256, ax[2], ax[3]])
            plt.title('estimated normal pdf of %sdense obejcts' % outlier_type)
            plt.show()

        return rv

    def get_unaries(self, data, mask, models, params):
        rv_heal = models['rv_heal']
        rv_hyper = models['rv_hyper']
        rv_hypo = models['rv_hypo']
        # mu_heal = models['mu_heal']
        mu_heal = rv_heal.mean()

        if params['erode_mask']:
            if data.ndim == 3:
                mask = tools.eroding3D(mask, skimor.disk(5), slicewise=True)
            else:
                mask = skimor.binary_erosion(mask, np.ones((5, 5)))

        unaries_healthy = - rv_heal.logpdf(data) * mask
        if params['unaries_as_cdf']:
            unaries_hyper = - np.log(rv_hyper.cdf(data) * rv_heal.pdf(mu_heal)) * mask
            # removing zeros with second lowest value so the log(0) wouldn't throw a warning -
            tmp = 1 - rv_hypo.cdf(data)
            values = np.unique(tmp)
            tmp = np.where(tmp == 0, values[1], tmp)
            #-
            unaries_hypo = - np.log(tmp * rv_heal.pdf(mu_heal)) * mask
            unaries_hypo = np.where(np.isnan(unaries_hypo), 0, unaries_hypo)
        else:
            unaries_hyper = - rv_hyper.logpdf(data) * mask
            unaries_hypo = - rv_hypo.logpdf(data) * mask

        unaries = np.dstack((unaries_hypo.reshape(-1, 1), unaries_healthy.reshape(-1, 1), unaries_hyper.reshape(-1, 1)))
        unaries = unaries.astype(np.int32)

        # slice = 17
        # # slice = 6
        # plt.figure()
        # plt.imshow(unaries_hypo.reshape(data.shape)[slice,:,:], 'gray')
        # while True:
        #     pt = plt.ginput(1)
        #     print pt[0][1], pt[0][0], ' - ',
        #     print 'hypo = ', unaries_hypo.reshape(data.shape)[slice, int(pt[0][1]), int(pt[0][0])],
        #     print 'heal = ', unaries_healthy.reshape(data.shape)[slice, int(pt[0][1]), int(pt[0][0])],
        #     print 'hyper = ', unaries_hyper.reshape(data.shape)[slice, int(pt[0][1]), int(pt[0][0])],
        #     print ', int = ', data[slice, int(pt[0][1]), int(pt[0][0])]
        #
        # plt.figure()
        # plt.subplot(221), plt.imshow(unaries_hypo.reshape(data.shape)[6,:,:], 'gray'), plt.colorbar()
        # plt.subplot(222), plt.imshow(unaries_hyper.reshape(data.shape)[6,:,:], 'gray'), plt.colorbar()
        # plt.subplot(223), plt.imshow(unaries_healthy.reshape(data.shape)[6,:,:], 'gray'), plt.colorbar()
        # plt.show()

        # self.params['show_unaries'] = True
        if self.params['show_unaries']:
            ints = data[np.nonzero(mask)]
            hist, bins = skiexp.histogram(ints, nbins=256)
            x = np.arange(0, 255, 0.01)
            healthy = rv_heal.pdf(x)
            if params['unaries_as_cdf']:
                hypo = (1 - rv_hypo.cdf(x)) * rv_heal.pdf(mu_heal)
                hyper = rv_hyper.cdf(x) * rv_heal.pdf(mu_heal)
            else:
                hypo = rv_hypo.pdf(x)
                hyper = rv_hyper.pdf(x)

            plt.figure()
            plt.subplot(211)
            plt.plot(bins, hist)
            ax = plt.axis()
            plt.axis([0, 256, ax[2], ax[3]])
            plt.title('histogram of input data')
            plt.subplot(212)
            plt.plot(x, hypo, 'm')
            plt.hold(True)
            plt.plot(x, healthy, 'g')
            plt.plot(x, hyper, 'r')
            ax = plt.axis()
            plt.axis([0, 256, ax[2], ax[3]])
            plt.title('histogram of input data')
            plt.legend(['hypodense pdf', 'healthy pdf', 'hyperdense pdf'])
            plt.show()

        return unaries

    # def mayavi_visualization(self, res):
    #     ### Read the data in a numpy 3D array ##########################################
    #     #parenchym = np.logical_or(liver, vessels)
    #     parenchym = res > 0
    #     hypodense = res == 0
    #     hyperdense = res == 2
    #     data = np.where(parenchym, 1, 0)
    #     data = data.T
    #
    #     src = mlab.pipeline.scalar_field(data)
    #     src.spacing = [1, 1, 1]
    #
    #     data2 = np.where(hypodense, 1, 0)
    #     # data2 = hypodense
    #     data2 = data2.T
    #     src2 = mlab.pipeline.scalar_field(data2)
    #     src2.spacing = [1, 1, 1]
    #
    #     data3 = np.where(hyperdense, 1, 0)
    #     # data3 = hyperdense
    #     data3 = data3.T
    #     src3 = mlab.pipeline.scalar_field(data3)
    #     src3.spacing = [1, 1, 1]
    #
    #     #contours 6 ... cevy
    #     #contours 3 ... jatra
    #     #contours 10 ... jatra a cevy
    #     #mlab.pipeline.iso_surface(src, contours=3, opacity=0.1)
    #     #mlab.pipeline.iso_surface(src, contours=2, opacity=0.2)
    #     mlab.pipeline.iso_surface(src, contours=2, opacity=0.2, color=(0, 1, 0))
    #     mlab.pipeline.iso_surface(src2, contours=2, opacity=0.2, color=(1, 0, 0))
    #     mlab.pipeline.iso_surface(src3, contours=2, opacity=0.2, color=(0, 0, 1))
    #
    #     mlab.show()

    def get_compactness(self, labels):
        nlabels = labels.max() + 1
        # nlabels = len(np.unique(labels)) - 1
        eccs = np.zeros(nlabels)

        for lab in range(nlabels):
            obj = labels == lab
            if labels.ndim == 2:
                strel = np.ones((3, 3), dtype=np.bool)
                obj_c = skimor.binary_closing(obj, strel)
                if obj_c.sum() >= obj.sum():
                    obj = obj_c
            else:
                strel = np.ones((3, 3, 3), dtype=np.bool)
                obj_c = tools.closing3D(obj, strel)
                if obj_c.sum() >= obj.sum():
                    obj = obj_c

            if labels.ndim == 2:
                ecc = skimea.regionprops(obj)[0]['eccentricity']
            else:
                ecc = tools.get_zunics_compatness(obj)
                # if np.isnan(ecc):
                #     ecc = 0
            eccs[lab] = ecc

        return eccs

    def get_areas(self, labels):
        nlabels = labels.max() + 1
        areas = np.zeros(nlabels)
        for i in range(nlabels):
            obj = labels == i
            areas[i] = obj.sum()

        return areas

    # def filter_objects(self, feature_v, features, params):
    #     min_area = params['min_area']
    #     min_comp = params['min_compactness']
    #
    #     obj_ok = np.zeros(feature_v.shape, dtype=np.bool)
    #     area_idx = -1
    #     comp_idx = -1
    #     try:
    #         area_idx = features.index('area')
    #     except ValueError:
    #         pass
    #     try:
    #         comp_idx = features.index('compactness')
    #     except ValueError:
    #         pass
    #
    #     if area_idx != -1:
    #         obj_ok[:, area_idx] = feature_v[:, area_idx] > min_area
    #
    #     if comp_idx != -1:
    #         obj_ok[:, comp_idx] = feature_v[:, comp_idx] > min_comp
    #
    #     return obj_ok

    def smooth_data(self, data):
        if data is not None:
            # smoothing data
            print 'smoothing data...'
            if self.params['smoothing'] == 1:
                data = skifil.gaussian_filter(self.data, sigma=self.params['sigma'])
            elif self.params['smoothing'] == 2:
                data = tools.smoothing_bilateral(data, sigma_space=self.params['sigma_spatial'], sigma_color=self.params['sigma_range'], sliceId=0)
            elif self.params['smoothing'] == 3:
                data = tools.smoothing_tv(data, weight=self.params['weight'], sliceId=0)
            else:
                print '\tcurrently switched off'

        return data

    def calculate_intensity_models(self, data, mask):
        print 'calculating intensity models...'
        # liver pdf ------------
        rv_heal = self.estimate_healthy_pdf(data, mask, self.params)
        print '\tliver pdf: mu = ', rv_heal.mean(), ', sigma = ', rv_heal.std()
        # hypodense pdf ------------
        rv_hypo = self.estimate_outlier_pdf(data, mask, rv_heal, 'hypo', self.params)
        print '\thypodense pdf: mu = ', rv_hypo.mean(), ', sigma = ', rv_hypo.std()
        # hyperdense pdf ------------
        rv_hyper = self.estimate_outlier_pdf(data, mask, rv_heal, 'hyper', self.params)
        print '\thyperdense pdf: mu = ', rv_hyper.mean(), ', sigma = ', rv_hyper.std()

        models = dict()
        models['rv_heal'] = rv_heal
        models['rv_hypo'] = rv_hypo
        models['rv_hyper'] = rv_hyper

        return models

    def objects_filtration(self, selected_labels=None,
                           area=None,#min_area=0, max_area=np.Infinity,
                           density=None,#min_density=-np.Infinity, max_density=np.Infinity):
                           compactness=None):

        if self.actual_data.lesions is not None:
            lesions = self.actual_data.lesions[:]  # copy of the list
        else:
            return

        if area is not None:
            lesions = [x for x in lesions if area[0] <= x.area <= area[1] or x.priority == Lesion.PRIORITY_HIGH]

        if density is not None:
            lesions = [x for x in lesions if density[0] <= x.mean_density <= density[1] or x.priority == Lesion.PRIORITY_HIGH]

        if compactness is not None:
            if compactness > 1:
                compactness = float(compactness) / self.params['compactness_step']
            lesions = [x for x in lesions if compactness <= x.compactness or x.priority == Lesion.PRIORITY_HIGH]

        # geting labels of filtered objects
        self.filtered_idxs = [x.label for x in lesions]

        if selected_labels is not None:
            self.filtered_idxs = np.intersect1d(self.filtered_idxs, selected_labels)

        self.actual_data.labels_filt = np.where(self.actual_data.labels > self.params['bgd_label'], self.params['healthy_label'], self.params['bgd_label'])
        is_in = np.in1d(self.actual_data.labels, self.filtered_idxs).reshape(self.actual_data.labels.shape)
        self.actual_data.labels_filt = np.where(is_in, self.actual_data.labels, self.actual_data.labels_filt)
        pass

    def run(self):
        # slice_idx = self.params['slice_idx']
        alpha = self.params['alpha']
        beta = self.params['beta']
        hypo_lab = self.params['hypo_label']
        hyper_lab = self.params['hyper_label']

        # if not fname:
        #     _, self.data, self.mask = self.load_data(slice_idx)
        # else:
        #     self.data, self.mask, self.voxel_size = self.load_pickle_data(self.fname, self.params, slice_idx)
        # self.data = self.data.astype(np.uint8)

        # TumorVisualiser.run(data, mask, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)

        # print 'estimating number of clusters ...',
        # print '\tcurrently switched off'
        # d = scindiint.zoom(data_o, 0.5)
        # m = skitra.resize(mask, np.array(mask.shape) * 0.5).astype(np.bool)
        # ints = d[np.nonzero(m)]
        # ints = ints.reshape((ints.shape[0], 1))
        # for i in range(2, 5):
        #     kmeans_model = KMeans(n_clusters=i, n_init=1).fit(ints)
        #     labels = kmeans_model.labels_
        #
        #     sc = metrics.silhouette_score(ints, labels, metric='euclidean')
        #     print '\tn_clusters = %i, score = %1.3f (best score=1, worse score=-1)' % (i, sc)

        # data_o = cv2.imread('/home/tomas/Dropbox/images/medicine/hypodense_bad2.png', 0).astype(np.float)

        # data_s = skifil.gaussian_filter(data_o, sigma)
        #
        # #imd = np.absolute(im - ims)
        # data_d = data_o - data_s
        #
        # data_d = np.where(data_d < 0, 0, data_d)

        # zooming the data
        print 'rescaling data ...',
        if self.params['zoom']:
            self.actual_data.data = self.data_zoom(self.actual_data.data, self.actual_data.voxel_size, self.params['working_voxel_size_mm'])
            self.actual_data.mask = self.data_zoom(self.actual_data.mask, self.actual_data.voxel_size, self.params['working_voxel_size_mm'])
        else:
            data = tools.resize3D(self.actual_data.data, self.params['scale'], sliceId=0)
            mask = tools.resize3D(self.actual_data.mask, self.params['scale'], sliceId=0)
        print 'ok'
        # data = data.astype(np.uint8)

        # calculating intensity models if necesarry
        print 'estimating color models ...'
        if not self.models:
            self.models = self.calculate_intensity_models(data, mask)
            print 'ok'
        else:
            print 'already done'

        print 'calculating unary potentials ...',
        self.status_bar.showMessage('Calculating unary potentials...')
        # create unaries
        # unaries = data_d
        # # as we convert to int, we need to multipy to get sensible values
        # unaries = (1 * np.dstack([unaries, -unaries]).copy("C")).astype(np.int32)
        self.unaries = beta * self.get_unaries(data, mask, self.models, self.params)
        n_labels = self.unaries.shape[2]
        print 'ok'

        print 'calculating pairwise potentials ...',
        self.status_bar.showMessage('Calculating pairwise potentials...')
        # create potts pairwise
        self.pairwise = -alpha * np.eye(n_labels, dtype=np.int32)
        print 'ok'

        print 'deriving graph edges ...',
        self.status_bar.showMessage('Deriving graph edges...')
        # use the gerneral graph algorithm
        # first, we construct the grid graph
        inds = np.arange(data.size).reshape(data.shape)
        if data.ndim == 2:
            horz = np.c_[inds[:, :-1].ravel(), inds[:, 1:].ravel()]
            vert = np.c_[inds[:-1, :].ravel(), inds[1:, :].ravel()]
            self.edges = np.vstack([horz, vert]).astype(np.int32)
        elif data.ndim == 3:
            # horz = np.c_[inds[:, :-1].ravel(), inds[:, 1:].ravel()]
            # vert = np.c_[inds[:-1, :].ravel(), inds[1:, :].ravel()]
            horz = np.c_[inds[:, :, :-1].ravel(), inds[:, :, 1:].ravel()]
            vert = np.c_[inds[:, :-1, :].ravel(), inds[:, 1:, :].ravel()]
            dept = np.c_[inds[:-1, :, :].ravel(), inds[1:, :, :].ravel()]
            self.edges = np.vstack([horz, vert, dept]).astype(np.int32)
        # deleting edges with nodes outside the mask
        nodes_in = np.ravel_multi_index(np.nonzero(mask), data.shape)
        rows_inds = np.in1d(self.edges, nodes_in).reshape(self.edges.shape).sum(axis=1) == 2
        self.edges = self.edges[rows_inds, :]
        print 'ok'

        # plt.show()

        # un = unaries[:, :, 0].reshape(data_o.shape)
        # un = unaries[:, :, 1].reshape(data_o.shape)
        # un = unaries[:, :, 2].reshape(data_o.shape)
        # py3DSeedEditor.py3DSeedEditor(un).show()

        print 'calculating graph cut ...',
        self.status_bar.showMessage('Calculating graph cut...')
        # we flatten the unaries
        # result_graph = pygco.cut_from_graph(edges, unaries.reshape(-1, 2), pairwise)
        # print 'tu: ', unaries.reshape(-1, n_labels).shape
        result_graph = pygco.cut_from_graph(self.edges, self.unaries.reshape(-1, n_labels), self.pairwise)
        labels = result_graph.reshape(data.shape) + 1  # +1 to shift the first class to label number 1

        labels = np.where(mask, labels, self.params['bgd_label'])

        # zooming to the original size
        if self.params['zoom']:
            self.actual_data.labels = self.zoom_to_shape(labels, self.actual_data.orig_shape)
        else:
            # self.actual_data.labels = tools.resize3D(labels, scale=1. / self.params['scale'], sliceId=0)
            # self.actual_data.labels2 = skitra.resize(labels, self.actual_data.orig_shape, mode='nearest', preserve_range=True).astype(np.int64)
            self.actual_data.labels = tools.resize3D(labels, shape=self.actual_data.orig_shape, sliceId=0)#.astype(np.int64)

        print 'ok'
        self.status_bar.showMessage('Done')

        # debug visualization
        # self.viewer = Viewer_3D.Viewer_3D(self.res, range=True)
        # self.viewer.show()

        print 'extracting objects ...',
        self.status_bar.showMessage('Extracting objects ...'),
        labels_tmp = np.where(self.actual_data.labels == self.params['healthy_label'], self.params['bgd_label'], self.actual_data.labels)  # because we can set only one label as bgd
        self.actual_data.objects = skimea.label(labels_tmp, background=self.params['bgd_label'])
        self.actual_data.lesions = Lesion.extract_lesions(self.actual_data.objects, self.actual_data.data)
        # areas = [x.area for x in self.actual_data.lesions]
        self.status_bar.showMessage('Done')
        print 'ok'

        print 'initial filtration ...',
        self.objects_filtration(area=(self.params['min_area'], self.params['max_area']),
                                density=(self.params['min_density'], self.params['max_density']),
                                compactness=self.params['min_compactness'])
        print 'ok'
        self.status_bar.showMessage('Done')


        # TumorVisualiser.run(self.data, self.res, self.params['healthy_label'], self.params['hypo_label'], self.params['hyper_label'], slice_axis=0)

        # mayavi_visualization(res)

#-------------------------------------------------------------
if __name__ == '__main__':

    fname = ''

    # 2 hypo, 1 on the border --------------------
    # arterial 0.6mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_183_46324212_arterial_0.6_B30f-.pklz'
    # venous 0.6mm - good
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_183_46324212_venous_0.6_B20f-.pklz'
    # venous 5mm - ok, but wrong approach
    # fname = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_183_46324212_venous_5.0_B30f-.pklz'

    # hypo in venous -----------------------
    # arterial - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_186_49290986_venous_0.6_B20f-.pklz'
    # venous - good
    fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_186_49290986_arterial_0.6_B20f-.pklz'

    # hyper, 1 on the border -------------------
    # arterial 0.6mm - not that bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_239_61293268_DE_Art_Abd_0.75_I26f_M_0.5-.pklz'
    # venous 5mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_239_61293268_DE_Ven_Abd_0.75_I26f_M_0.5-.pklz'

    # shluk -----------------
    # arterial 5mm
    # fname = '/home/tomas/Data/liver_segmentation/tryba/data_other/org-exp_180_49509315_arterial_5.0_B30f-.pklz'
    # fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_180_49509315_arterial_0.6_B20f-.pklz'

    # targeted
    # arterial 0.6mm - bad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_238_54280551_Abd_Arterial_0.75_I26f_3-.pklz'
    # venous 0.6mm - b  ad
    # fname = '/home/tomas/Data/liver_segmentation_06mm/hyperdenzni/org-exp_238_54280551_Abd_Venous_0.75_I26f_3-.pklz'

    cc = Computational_core(fname)
    cc.run()


__author__ = 'tomas'


import numpy as np
import matplotlib.pyplot as plt
import scipy.ndimage.morphology as scindimor
import scipy.ndimage.measurements as scindimea
import scipy.ndimage.interpolation as scindiint
import scipy.ndimage as scindi
import scipy.stats as scista

import skimage.segmentation as skiseg
import skimage.morphology as skimor
import skimage.filter as skifil
import skimage.exposure as skiexp
import skimage.measure as skimea
import skimage.transform as skitra

import cv2
import pygco

import tools
import py3DSeedEditor
# from mayavi import mlab

import TumorVisualiser

from sklearn import metrics
from sklearn.cluster import KMeans

import pickle

import Viewer_3D

import cv2

import Data
import Lesion

import logging
logger = logging.getLogger(__name__)
logging.basicConfig()


def data_zoom(data, voxelsize_mm, working_voxelsize_mm):
    zoom = voxelsize_mm / (1.0 * working_voxelsize_mm)
    data_res = scindi.zoom(data, zoom, mode='nearest', order=1).astype(np.int16)
    return data_res

def zoom_to_shape(data, shape):
    zoom = np.array(shape, dtype=np.float) / np.array(data.shape, dtype=np.float)
    data_res = scindi.zoom(data, zoom, mode='nearest', order=1).astype(np.int16)
    return data_res

def estimate_healthy_pdf(data, mask, params):
    perc = params['perc']
    k_std_l = params['k_std_h']
    simple_estim = params['healthy_simple_estim']
    # show_me = params['show_healthy_pdf_estim']

    # data = tools.smoothing_tv(data.astype(np.uint8), tv_weight)
    ints = data[np.nonzero(mask)]
    hist, bins = skiexp.histogram(ints, nbins=256)
    if simple_estim:
        mu, sigma = scista.norm.fit(ints)
    else:
        ints = data[np.nonzero(mask)]

        n_pts = mask.sum()
        perc_in = n_pts * perc / 100

        peak_idx = np.argmax(hist)
        n_in = hist[peak_idx]
        win_width = 0

        while n_in < perc_in:
            win_width += 1
            n_in = hist[peak_idx - win_width:peak_idx + win_width].sum()

        idx_start = bins[peak_idx - win_width]
        idx_end = bins[peak_idx + win_width]
        inners_m = np.logical_and(ints > idx_start, ints < idx_end)
        inners = ints[np.nonzero(inners_m)]

        # liver pdf -------------
        mu = bins[peak_idx]# + params['hack_healthy_mu']
        sigma = k_std_l * np.std(inners)# + params['hack_healthy_sigma']

    mu = int(mu)
    sigma = int(sigma)
    rv = scista.norm(mu, sigma)

    # if show_me:
    #     plt.figure()
    #     plt.subplot(211)
    #     plt.plot(bins, hist)
    #     plt.title('histogram with max peak')
    #     plt.hold(True)
    #     plt.plot([mu, mu], [0, hist.max()], 'g')
    #     ax = plt.axis()
    #     plt.axis([0, 256, ax[2], ax[3]])
    #     # plt.subplot(212), plt.plot(bins, rv_l.pdf(bins), 'g')
    #     x = np.arange(0, 256, 0.1)
    #     plt.subplot(212), plt.plot(x, rv.pdf(x), 'g')
    #     plt.hold(True)
    #     plt.plot(mu, rv.pdf(mu), 'go')
    #     ax = plt.axis()
    #     plt.axis([0, 256, ax[2], ax[3]])
    #     plt.title('estimated normal pdf of healthy parenchym')
    #     # plt.show()

    return rv

def estimate_outlier_pdf(data, mask, rv_healthy, outlier_type, params):
    prob_w = params['prob_w']
    # show_me = params['show_outlier_pdf_estim']

    # if outlier_type == 'hypo':
    #     hack_mu = params['hack_hypo_mu']
    #     hack_sigma = params['hack_hypo_sigma']
    # elif outlier_type == 'hyper':
    #     hack_mu = params['hack_hyper_mu']
    #     hack_sigma = params['hack_hyper_sigma']

    probs = rv_healthy.pdf(data) * mask
    # hist, bins = skiexp.histogram(probs, nbins=100)
    # plt.figure()
    # plt.plot(bins, hist)
    # plt.show()

    max_prob = rv_healthy.pdf(rv_healthy.mean())
    # mean_prob = np.mean(probs[np.nonzero(mask)])
    print 'max_prob = %.3f' % max_prob
    # print 'mean prob = %.3f' % mean_prob
    prob_t = prob_w * max_prob

    ints_out_m = probs < prob_t * mask

    # TumorVisualiser.run(data, ints_out_m, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)

    ints_out = data[np.nonzero(ints_out_m)]
    hist, bins = skiexp.histogram(ints_out, nbins=256)

    if outlier_type == 'hypo':
        ints = ints_out[np.nonzero(ints_out < rv_healthy.mean())]
        # m = ints_out_m * (data < rv_healthy.mean())
        # TumorVisualiser.run(data, m, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)
    elif outlier_type == 'hyper':
        ints = ints_out[np.nonzero(ints_out > rv_healthy.mean())]
        # m = ints_out_m * (data > rv_healthy.mean())
        # TumorVisualiser.run(data, m, params['healthy_label'], params['hypo_label'], params['hyper_label'], slice_axis=0, disp_smoothed=True)
    else:
        print 'Wrong outlier specification.'
        return

    mu, sigma = scista.norm.fit(ints)

    # hack for moving pdfs of hypo and especially of hyper furhter from the pdf of healthy parenchyma
    # mu += hack_mu
    # sigma += hack_sigma
    #-----------

    mu = int(mu)
    sigma = int(sigma)
    rv = scista.norm(mu, sigma)

    # if show_me:
    #     plt.figure()
    #     plt.subplot(211)
    #     plt.plot(bins, hist)
    #     plt.title('histogram with max peak')
    #     plt.hold(True)
    #     plt.plot([mu, mu], [0, hist.max()], 'g')
    #     ax = plt.axis()
    #     plt.axis([0, 256, ax[2], ax[3]])
    #     # plt.subplot(212), plt.plot(bins, rv_l.pdf(bins), 'g')
    #     x = np.arange(0, 256, 0.1)
    #     plt.subplot(212), plt.plot(x, rv.pdf(x), 'g')
    #     plt.hold(True)
    #     plt.plot(mu, rv.pdf(mu), 'go')
    #     ax = plt.axis()
    #     plt.axis([0, 256, ax[2], ax[3]])
    #     plt.title('estimated normal pdf of %sdense obejcts' % outlier_type)
    #     plt.show()

    return rv

def get_unaries(data, mask, models, params):
    rv_heal = models['heal']
    rv_hyper = models['hyper']
    rv_hypo = models['hypo']
    # mu_heal = models['mu_heal']
    mu_heal = rv_heal.mean()

    # if params['erode_mask']:
    #     if data.ndim == 3:
    #         mask = tools.eroding3D(mask, skimor.disk(5), slicewise=True)
    #     else:
    #         mask = skimor.binary_erosion(mask, np.ones((5, 5)))

    unaries_healthy = - rv_heal.logpdf(data) * mask
    if params['unaries_as_cdf']:
        unaries_hyper = - np.log(rv_hyper.cdf(data) * rv_heal.pdf(mu_heal)) * mask
        # removing zeros with second lowest value so the log(0) wouldn't throw a warning -
        tmp = 1 - rv_hypo.cdf(data)
        values = np.unique(tmp)
        tmp = np.where(tmp == 0, values[1], tmp)
        #-
        unaries_hypo = - np.log(tmp * rv_heal.pdf(mu_heal)) * mask
        unaries_hypo = np.where(np.isnan(unaries_hypo), 0, unaries_hypo)
    else:
        unaries_hyper = - rv_hyper.logpdf(data) * mask
        unaries_hypo = - rv_hypo.logpdf(data) * mask

    unaries = np.dstack((unaries_hypo.reshape(-1, 1), unaries_healthy.reshape(-1, 1), unaries_hyper.reshape(-1, 1)))
    unaries = unaries.astype(np.int32)

    # slice = 17
    # # slice = 6
    # plt.figure()
    # plt.imshow(unaries_hypo.reshape(data.shape)[slice,:,:], 'gray')
    # while True:
    #     pt = plt.ginput(1)
    #     print pt[0][1], pt[0][0], ' - ',
    #     print 'hypo = ', unaries_hypo.reshape(data.shape)[slice, int(pt[0][1]), int(pt[0][0])],
    #     print 'heal = ', unaries_healthy.reshape(data.shape)[slice, int(pt[0][1]), int(pt[0][0])],
    #     print 'hyper = ', unaries_hyper.reshape(data.shape)[slice, int(pt[0][1]), int(pt[0][0])],
    #     print ', int = ', data[slice, int(pt[0][1]), int(pt[0][0])]
    #
    # plt.figure()
    # plt.subplot(221), plt.imshow(unaries_hypo.reshape(data.shape)[6,:,:], 'gray'), plt.colorbar()
    # plt.subplot(222), plt.imshow(unaries_hyper.reshape(data.shape)[6,:,:], 'gray'), plt.colorbar()
    # plt.subplot(223), plt.imshow(unaries_healthy.reshape(data.shape)[6,:,:], 'gray'), plt.colorbar()
    # plt.show()

    # self.params['show_unaries'] = True
    # if params['show_unaries']:
    #     ints = data[np.nonzero(mask)]
    #     hist, bins = skiexp.histogram(ints, nbins=256)
    #     x = np.arange(0, 255, 0.01)
    #     healthy = rv_heal.pdf(x)
    #     if params['unaries_as_cdf']:
    #         hypo = (1 - rv_hypo.cdf(x)) * rv_heal.pdf(mu_heal)
    #         hyper = rv_hyper.cdf(x) * rv_heal.pdf(mu_heal)
    #     else:
    #         hypo = rv_hypo.pdf(x)
    #         hyper = rv_hyper.pdf(x)
    #
    #     plt.figure()
    #     plt.subplot(211)
    #     plt.plot(bins, hist)
    #     ax = plt.axis()
    #     plt.axis([0, 256, ax[2], ax[3]])
    #     plt.title('histogram of input data')
    #     plt.subplot(212)
    #     plt.plot(x, hypo, 'm')
    #     plt.hold(True)
    #     plt.plot(x, healthy, 'g')
    #     plt.plot(x, hyper, 'r')
    #     ax = plt.axis()
    #     plt.axis([0, 256, ax[2], ax[3]])
    #     plt.title('histogram of input data')
    #     plt.legend(['hypodense pdf', 'healthy pdf', 'hyperdense pdf'])
    #     plt.show()

    return unaries

# def mayavi_visualization(self, res):
#     ### Read the data in a numpy 3D array ##########################################
#     #parenchym = np.logical_or(liver, vessels)
#     parenchym = res > 0
#     hypodense = res == 0
#     hyperdense = res == 2
#     data = np.where(parenchym, 1, 0)
#     data = data.T
#
#     src = mlab.pipeline.scalar_field(data)
#     src.spacing = [1, 1, 1]
#
#     data2 = np.where(hypodense, 1, 0)
#     # data2 = hypodense
#     data2 = data2.T
#     src2 = mlab.pipeline.scalar_field(data2)
#     src2.spacing = [1, 1, 1]
#
#     data3 = np.where(hyperdense, 1, 0)
#     # data3 = hyperdense
#     data3 = data3.T
#     src3 = mlab.pipeline.scalar_field(data3)
#     src3.spacing = [1, 1, 1]
#
#     #contours 6 ... cevy
#     #contours 3 ... jatra
#     #contours 10 ... jatra a cevy
#     #mlab.pipeline.iso_surface(src, contours=3, opacity=0.1)
#     #mlab.pipeline.iso_surface(src, contours=2, opacity=0.2)
#     mlab.pipeline.iso_surface(src, contours=2, opacity=0.2, color=(0, 1, 0))
#     mlab.pipeline.iso_surface(src2, contours=2, opacity=0.2, color=(1, 0, 0))
#     mlab.pipeline.iso_surface(src3, contours=2, opacity=0.2, color=(0, 0, 1))
#
#     mlab.show()

def get_compactness(labels):
    nlabels = labels.max() + 1
    # nlabels = len(np.unique(labels)) - 1
    eccs = np.zeros(nlabels)

    for lab in range(nlabels):
        obj = labels == lab
        if labels.ndim == 2:
            strel = np.ones((3, 3), dtype=np.bool)
            obj_c = skimor.binary_closing(obj, strel)
            if obj_c.sum() >= obj.sum():
                obj = obj_c
        else:
            strel = np.ones((3, 3, 3), dtype=np.bool)
            obj_c = tools.closing3D(obj, strel)
            if obj_c.sum() >= obj.sum():
                obj = obj_c

        if labels.ndim == 2:
            ecc = skimea.regionprops(obj)[0]['eccentricity']
        else:
            ecc = tools.get_zunics_compatness(obj)
            # if np.isnan(ecc):
            #     ecc = 0
        eccs[lab] = ecc

    return eccs

def get_areas(labels):
    nlabels = labels.max() + 1
    areas = np.zeros(nlabels)
    for i in range(nlabels):
        obj = labels == i
        areas[i] = obj.sum()

    return areas

def smooth_data(data, params):
    if data is not None:
        # smoothing data
        print 'smoothing data...'
        if params['smoothing'] == 1:
            data = skifil.gaussian_filter(data, sigma=params['sigma'])
        elif params['smoothing'] == 2:
            data = tools.smoothing_bilateral(data, sigma_space=params['sigma_spatial'], sigma_color=params['sigma_range'], sliceId=0)
        elif params['smoothing'] == 3:
            data = tools.smoothing_tv(data, weight=params['weight'], sliceId=0)
        else:
            print '\tcurrently switched off'

    return data

def calculate_intensity_models(data, mask, params):
    print 'calculating intensity models...'
    # liver pdf ------------
    rv_heal = estimate_healthy_pdf(data, mask, params)
    print '\tliver pdf: mu = ', rv_heal.mean(), ', sigma = ', rv_heal.std()
    # hypodense pdf ------------
    rv_hypo = estimate_outlier_pdf(data, mask, rv_heal, 'hypo', params)
    print '\thypodense pdf: mu = ', rv_hypo.mean(), ', sigma = ', rv_hypo.std()
    # hyperdense pdf ------------
    rv_hyper = estimate_outlier_pdf(data, mask, rv_heal, 'hyper', params)
    print '\thyperdense pdf: mu = ', rv_hyper.mean(), ', sigma = ', rv_hyper.std()

    models = dict()
    models['heal'] = rv_heal
    models['hypo'] = rv_hypo
    models['hyper'] = rv_hyper

    return models

def objects_filtration(data, params, selected_labels=None, area=None, density=None, compactness=None):
    if data.lesions is not None:
        lesions = data.lesions[:]  # copy of the list
    else:
        return

    if area is None:
        area = [params['min_area'], params['max_area']]
    if density is None:
        density = [params['min_density'], params['max_density']]
    if compactness is None:
        compactness = params['min_compactness']

    if area is not None:
        lesions = [x for x in lesions if area[0] <= x.area <= area[1] or x.priority == Lesion.PRIORITY_HIGH]

    if density is not None:
        lesions = [x for x in lesions if density[0] <= x.mean_density <= density[1] or x.priority == Lesion.PRIORITY_HIGH]

    if compactness is not None:
        if compactness > 1:
            compactness = float(compactness) / params['compactness_step']
        lesions = [x for x in lesions if compactness <= x.compactness or x.priority == Lesion.PRIORITY_HIGH]

    # geting labels of filtered objects
    filtered_lbls = [x.label for x in lesions]

    if selected_labels is not None:
        filtered_lbls = np.intersect1d(filtered_lbls, selected_labels)

    labels_filt_tmp = np.where(data.labels > params['bgd_label'], params['healthy_label'], params['bgd_label'])
    is_in = np.in1d(data.objects, filtered_lbls).reshape(data.labels.shape)
    data.labels_filt = np.where(is_in, data.labels, labels_filt_tmp)
    # print 'setting lesion_filt ...',
    data.lesions_filt = [x for x in data.lesions if x.label in filtered_lbls]
    # print 'done'

    return filtered_lbls

def run_mrf(data_o, params):
    # slice_idx = self.params['slice_idx']
    alpha = params['alpha']
    beta = params['beta']
    # hypo_lab = params['hypo_label']
    # hyper_lab = params['hyper_label']

    # zooming the data
    print 'rescaling data ...',
    if params['zoom']:
        data = data_zoom(data_o.data, data_o.voxel_size, params['working_voxel_size_mm'])
        mask = data_zoom(data_o.mask, data_o.voxel_size, params['working_voxel_size_mm'])
    else:
        data = tools.resize3D(data_o.data, params['scale'], sliceId=0)
        mask = tools.resize3D(data_o.mask, params['scale'], sliceId=0)
    print 'ok'
    # data = data.astype(np.uint8)

    # calculating intensity models if necesarry
    print 'estimating color models ...'
    if data_o.models is None:
        data_o.models = calculate_intensity_models(data, mask, params)
        print 'ok'
    else:
        print 'already done'

    print 'calculating unary potentials ...',
    # self.status_bar.showMessage('Calculating unary potentials...')
    # create unaries
    # # as we convert to int, we need to multipy to get sensible values
    # unaries = (1 * np.dstack([unaries, -unaries]).copy("C")).astype(np.int32)
    unaries = beta * get_unaries(data, mask, data_o.models, params)
    n_labels = unaries.shape[2]
    print 'ok'

    print 'calculating pairwise potentials ...',
    # self.status_bar.showMessage('Calculating pairwise potentials...')
    # create potts pairwise
    pairwise = -alpha * np.eye(n_labels, dtype=np.int32)
    print 'ok'

    print 'deriving graph edges ...',
    # self.status_bar.showMessage('Deriving graph edges...')
    # use the gerneral graph algorithm
    # first, we construct the grid graph
    inds = np.arange(data.size).reshape(data.shape)
    if data.ndim == 2:
        horz = np.c_[inds[:, :-1].ravel(), inds[:, 1:].ravel()]
        vert = np.c_[inds[:-1, :].ravel(), inds[1:, :].ravel()]
        edges = np.vstack([horz, vert]).astype(np.int32)
    elif data.ndim == 3:
        horz = np.c_[inds[:, :, :-1].ravel(), inds[:, :, 1:].ravel()]
        vert = np.c_[inds[:, :-1, :].ravel(), inds[:, 1:, :].ravel()]
        dept = np.c_[inds[:-1, :, :].ravel(), inds[1:, :, :].ravel()]
        edges = np.vstack([horz, vert, dept]).astype(np.int32)
    # deleting edges with nodes outside the mask
    nodes_in = np.ravel_multi_index(np.nonzero(mask), data.shape)
    rows_inds = np.in1d(edges, nodes_in).reshape(edges.shape).sum(axis=1) == 2
    edges = edges[rows_inds, :]
    print 'ok'

    # plt.show()

    # un = unaries[:, :, 0].reshape(data_o.shape)
    # un = unaries[:, :, 1].reshape(data_o.shape)
    # un = unaries[:, :, 2].reshape(data_o.shape)
    # py3DSeedEditor.py3DSeedEditor(un).show()

    print 'calculating graph cut ...',
    # self.status_bar.showMessage('Calculating graph cut...')
    result_graph = pygco.cut_from_graph(edges, unaries.reshape(-1, n_labels), pairwise)
    labels = result_graph.reshape(data.shape) + 1  # +1 to shift the first class to label number 1

    labels = np.where(mask, labels, params['bgd_label'])

    # zooming to the original size
    if params['zoom']:
        data_o.labels = zoom_to_shape(labels, data_o.orig_shape)
    else:
        data_o.labels = tools.resize3D(labels, shape=data_o.orig_shape, sliceId=0)#.astype(np.int64)

    print 'ok'
    # self.status_bar.showMessage('Done')

    # debug visualization
    # self.viewer = Viewer_3D.Viewer_3D(self.res, range=True)
    # self.viewer.show()

    print 'extracting objects ...',
    # self.status_bar.showMessage('Extracting objects ...'),
    labels_tmp = np.where(data_o.labels == params['healthy_label'], params['bgd_label'], data_o.labels)  # because we can set only one label as bgd
    data_o.objects = skimea.label(labels_tmp, background=params['bgd_label'])
    data_o.lesions = Lesion.extract_lesions(data_o.objects, data_o.data)
    # self.status_bar.showMessage('Done')
    print 'ok'

    # TumorVisualiser.run(self.data, self.res, self.params['healthy_label'], self.params['hypo_label'], self.params['hyper_label'], slice_axis=0)

    # mayavi_visualization(res)



from PyQt4.QtCore import *
from PyQt4.QtGui import *
import sys
import numpy as np


class MyTableModel(QAbstractTableModel):
    def __init__(self, objects, data, headerdata=None, parent=None, *args):
        QAbstractTableModel.__init__(self, parent, *args)

        if headerdata is None:
            headerdata = ['label', 'area', 'mean density', 'compactness']

        self.objects = objects  # lesion list
        self.headerdata = headerdata
        self.data_arr = data  # data -> must be here to be able to del an object

    def rowCount(self, parent=QModelIndex()):
        # Number of rows corresponds to the number of objects
        return len(self.objects)

    def columnCount(self, parent=QModelIndex()):
        # Number of columns corresponds to the number of features
        return len(self.headerdata)

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return QVariant()
        elif role != Qt.DisplayRole:
            return QVariant()
        data = (self.objects[index.row()].label, self.objects[index.row()].area, self.objects[index.row()].mean_density,
                self.objects[index.row()].compactness )
        if data[index.column()] is not None:
            if isinstance(data[index.column()], int):
                out = QVariant('%i' % data[index.column()])
            elif isinstance(data[index.column()], float):
                out = QVariant('%.3f' % data[index.column()])
            else:
                out = QVariant('x')
        else:
            out = QVariant(' ')
        return out

    def headerData(self, idx, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return QVariant(self.headerdata[idx])
        if orientation == Qt.Vertical and role == Qt.DisplayRole:
            return QVariant(idx)
        return QVariant()

    def removeRows(self, row, count=1, parent=QModelIndex()):
        self.beginRemoveRows(QModelIndex(), row, row + count - 1)
        # self.objects = self.objects[:row] + self.objects[row + count:]
        lbl = self.objects[row].label
        self.data_arr.labels = np.where(self.data_arr.labels == lbl, 0, self.data_arr.labels)
        self.objects.pop(row)
        self.endRemoveRows()
        print 'removed label', lbl
        # print self.labels
        return True


class MyWindow(QWidget):
    def __init__(self, objects, labels, header=None, *args):
        QWidget.__init__(self, *args)

        self.data_arr = objects
        self.labels = labels
        self.tablemodel = MyTableModel(self.data_arr, self.labels, header, self)
        self.tableview = QTableView()
        self.tableview.setModel(self.tablemodel)
        self.tableview.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableview.selectionModel().selectionChanged.connect(self.selection_changed)

        add_BTN = QPushButton('add')
        self.connect(add_BTN, SIGNAL('clicked()'), self.add_row)

        del_BTN = QPushButton('delete')
        self.connect(del_BTN, SIGNAL('clicked()'), self.del_row)

        layoutV = QVBoxLayout(self)
        layoutV.addWidget(self.tableview)

        layoutH = QHBoxLayout(self)
        layoutH.addWidget(add_BTN)
        layoutH.addWidget(del_BTN)

        btn_frame = QFrame()
        btn_frame.setLayout(layoutH)

        layoutV.addWidget(btn_frame)
        self.setLayout(layoutV)

    def selection_changed(self, selected, deselected):
        print selected
        print deselected
        print '--'
        indexes = self.tableview.selectionModel().selectedRows()
        sel_mod = self.tableview.selectionModel()
        print self.tableview.selectionBehavior()
        pass
    #     indexes = self.tableview.selectionModel().selectedRows()
    #     for index in indexes:
    #         print index.row()
            # self.tableview.selectRow(index.row())

    def add_row(self):
        # try:
        #     new_idx = self.data[:, 0].max() + 1
        #     self.data = np.vstack((self.data, np.array([new_idx, 0, 0])))
        # except ValueError:
        #     self.data = np.array([[1, 0, 0]])
        # self.tablemodel.data = self.data
        # self.tableview.model().layoutChanged.emit()
        print 'row added'

    def del_row(self):
        # self.data = self.data[:-1, :]
        # self.tablemodel.data = self.data
        # self.tableview.model().layoutChanged.emit()
        indexes = self.tableview.selectionModel().selectedRows()
        for i in reversed(indexes):
            self.tablemodel.removeRow(i.row())
            # print 'deleted row', i.row()


if __name__ == "__main__":
    import Lesion

    class Data(object):
        def __init__(self, labels):
            self.labels = labels

    labels = np.array([[1, 1, 0, 2, 0],
                       [1, 1, 0, 2, 0],
                       [0, 0, 0, 2, 0],
                       [3, 0, 4, 0, 5],
                       [3, 0, 4, 0, 0]], dtype=np.int)
    labels = np.dstack((labels, labels, labels))

    data = Data(labels)

    lesions = Lesion.extract_lesions(labels, data=data.labels)

    app = QApplication(sys.argv)
    w = MyWindow(lesions, data)
    w.show()
    sys.exit(app.exec_())


__author__ = 'tomas'

import sys

import skimage.io as skiio
import skimage.exposure as skiexp

import numpy as np

from PyQt4.QtGui import QApplication, QLabel, QPixmap, QImage
from PyQt4.QtCore import Qt, QSize


class SliceBox(QLabel):

    def __init__(self, img, mode='seeds'):

        QLabel.__init__(self)

        self.image = img
        self.imageshape = img.shape

        self.imagesize = QSize(self.imageshape[0], self.imageshape[1])

        self.qimage = QImage(self.image.repeat(4),
                     self.imageshape[1], self.imageshape[0],
                     QImage.Format_RGB32).scaled(self.imagesize, aspectRatioMode=Qt.KeepAspectRatio)
        self.setPixmap(QPixmap.fromImage(self.qimage))
        self.orig_pixmap = self.pixmap().copy()


    def resizeEvent(self, event):
        self.resizeSlice(self.width(), self.height())


    def resizeSlice(self, w, h):
        self.setPixmap(self.orig_pixmap.scaled(w, h, Qt.KeepAspectRatio, Qt.SmoothTransformation))


#-----------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------
fname = '/home/tomas/Dropbox/images/puma.png'
image = skiio.imread(fname, as_grey=True)
image = skiexp.rescale_intensity(image, (0,1), (0,255)).astype(np.uint8)

app = QApplication(sys.argv)
label = SliceBox(image)
label.show()

sys.exit(app.exec_())


__author__ = 'tomas'

import numpy as np
import Computational_core as gc

import scipy.ndimage.measurements as scindimea
import skimage.transform as skitra
from Computational_core import *


params = dict()
params['sigma'] = 10  # sigma for gaussian blurr
params['alpha'] = 3  # weightening parameter for pairwise term
params['beta'] = 1  # weightening parameter for unary term
params['perc'] = 0.3  # what portion of liver parenchym around peak is used to calculate std of liver normal pdf
params['k_std_h'] = 3  # weighting parameter for sigma of normal distribution of healthy parenchym
params['k_std_t'] = 3  # weighting parameter for sigma of normal distribution of tumor
params['tv_weight'] = 0.05  # weighting parameter for total variation filter
params['healthy_simple_estim'] = False  # simple healthy parenchym pdf estimation from all data
params['prob_w'] = 0.5  # prob_w * max_prob is a threshold for data that will be used for estimation of other pdfs

params['hack_hypo_mu'] = -0  # hard move of mean of hypodense pdf to the left
params['hack_hypo_sigma'] = 0  # hard widening of sigma of hypodense pdf
params['hack_hyper_mu'] = 5  # hard move of mean of hyperdense pdf to the right
params['hack_hyper_sigma'] = 5  # hard widening of sigma of hyperdense  pdf

params['show_healthy_pdf_estim'] = False
params['show_estimated_pdfs'] = True
params['show_outlier_pdf_estim'] = False

params['hypo_label'] = 0  # label of hypodense objects
params['hyper_label'] = 2  # label of hyperdense objects
params['healthy_label'] = 1  # label of hyalthy parenchyma
params['min_area'] = 20
params['min_compactness'] = 0.2

data = np.load('input_orig_data.npy')
data_sm = np.load('input_data.npy')
mask = np.load('mask.npy')

params['slice_idx'] = 33

# plt.figure()
# plt.subplot(221), plt.imshow(data[params['slice_idx'], :, :], 'gray'), plt.title('input_orig_data')
# plt.subplot(222), plt.imshow(data_sm[params['slice_idx'], :, :], 'gray'), plt.title('input_data')
# plt.subplot(224), plt.imshow(mask[params['slice_idx'], :, :], 'gray'), plt.title('mask')
# plt.show()

data = data[params['slice_idx'], :, :]
data_sm = data_sm[params['slice_idx'], :, :]
data_sm = skitra.resize(data_sm, data.shape)
mask = mask[params['slice_idx'], :, :]

# plt.figure()
# plt.subplot(121), plt.imshow(data, 'gray')
# plt.subplot(122), plt.imshow(mask, 'gray')
# plt.show()

# healthy pdf ------------
print 'estimating pdf of healthy parenchym...'
mu_h, sigma_h, rv_healthy = estimate_healthy_pdf(data, mask, params)
print 'liver pdf: mu = ', mu_h, ', sigma = ', sigma_h
mu_h_sm, sigma_h_sm, rv_healthy_sm = estimate_healthy_pdf(data_sm, mask, params)
print 'liver pdf: mu_sm = ', mu_h_sm, ', sigma_sm = ', sigma_h_sm


if data.ndim == 3:
    mask_e = tools.eroding3D(mask, skimor.disk(5))
else:
    mask_e = skimor.binary_erosion(mask, np.ones((5, 5)))
# mask_e = mask
unaries_healthy = - rv_healthy.logpdf(data) * mask_e
prob_healthy = rv_healthy.pdf(data) * mask_e

unaries_healthy_sm = - rv_healthy_sm.logpdf(data_sm) * mask_e
prob_healthy_sm = rv_healthy_sm.pdf(data_sm) * mask_e

# hypodense pdf ------------
print 'estimating pdf of hypodense objects...'
# mu_hypo, sigma_hypo, rv_hypo = estimate_hypo_pdf(data, mask, rv_healthy, show_me)
mu_hypo, sigma_hypo, rv_hypo = estimate_outlier_pdf(data, mask, rv_healthy, 'hypo', params)
print 'hypodense pdf: mu = ', mu_hypo, ', sigma= ', sigma_hypo

# hyperdense pdf ------------
print 'estimating pdf of hyperdense objects...'
# mu_hyper, sigma_hyper, rv_hyper = estimate_hyper_pdf(data, mask, rv_healthy, show_me)
mu_hyper, sigma_hyper, rv_hyper = estimate_outlier_pdf(data, mask, rv_healthy, 'hyper', params)
print 'hyperdense pdf: mu = ', mu_hyper, ', sigma= ', sigma_hyper

# PDF prob
prob_hyper_pdf = rv_hypo.pdf(data) * mask_e
prob_hypo_pdf = rv_hyper.pdf(data) * mask_e

# cumsum
x = np.arange(0, 255, 0.1)
hyper_cdf = rv_hyper.cdf(x) * rv_healthy.pdf(mu_h)
hypo_cdf = rv_hypo.cdf(x) * rv_healthy.pdf(mu_h)
hypo_cdf = hypo_cdf.max() - hypo_cdf

prob_hyper_cdf = rv_hyper.cdf(data) * rv_healthy.pdf(mu_h) * mask_e
prob_hypo_cdf = (1 - rv_hypo.cdf(data)) * rv_healthy.pdf(mu_h) * mask_e

# plt.figure()
# plt.plot(x, rv_healthy.pdf(x), 'g')
# plt.hold(True)
# plt.plot(x, hypo_cdf, 'b')
# plt.plot(x, hyper_cdf, 'r')
# plt.show()

# PDF versus CDF --------------------------------------------------------------------------------
plt.figure()
plt.subplot(321), plt.imshow(data, 'gray', interpolation='nearest'), plt.title('input')
plt.subplot(322), plt.imshow(prob_healthy, 'gray', interpolation='nearest'), plt.title('healthy')

plt.subplot(323), plt.imshow(prob_hyper_pdf, 'gray', interpolation='nearest'), plt.title('hypodense - pdf')
plt.subplot(324), plt.imshow(prob_hypo_pdf, 'gray', interpolation='nearest'), plt.title('hyperdense - pdf')

plt.subplot(325), plt.imshow(prob_hypo_cdf, 'gray', interpolation='nearest'), plt.title('hypodense - cdf')
plt.subplot(326), plt.imshow(prob_hyper_cdf, 'gray', interpolation='nearest'), plt.title('hyperdense - cdf')


# ploting GMM ------------------------------------
ints = data[np.nonzero(mask)]
hist, bins = skiexp.histogram(ints, nbins=256)
plt.figure()
plt.subplot(311)
plt.title('histogram of input data')
plt.plot(bins, hist)
ax = plt.axis()
plt.axis([0, 256, ax[2], ax[3]])

plt.subplot(312)
plt.title('GMM')
plt.plot(x, rv_hypo.pdf(x), 'b')
plt.plot(x, rv_healthy.pdf(x), 'g')
plt.plot(x, rv_hyper.pdf(x), 'r')
ax = plt.axis()
plt.axis([0, 256, ax[2], ax[3]])

x = np.arange(0, 255, 0.1)
healthy = rv_healthy.pdf(x)
hypo = (1 - rv_hypo.cdf(x)) * rv_healthy.pdf(mu_h)
hyper = rv_hyper.cdf(x) * rv_healthy.pdf(mu_h)

plt.subplot(313)
plt.title('cdf')
plt.plot(x, healthy, 'g')
plt.plot(x, hypo, 'b')
plt.plot(x, hyper, 'r')
ax = plt.axis()
plt.axis([0, 255, ax[2], ax[3]])


# plt.figure()
# plt.plot(x, healthy, 'g')
# plt.plot(x, hypo, 'b')
# plt.plot(x, hyper, 'r')
# ax = plt.axis()
# plt.axis([0, 255, ax[2], ax[3]])

plt.show()


__author__ = 'tomas'

import numpy as np
import Computational_core as gc

import scipy.ndimage.measurements as scindimea
from Computational_core import *


params = dict()
params['slice_idx'] = -1
params['sigma'] = 10  # sigma for gaussian blurr
params['alpha'] = 3  # weightening parameter for pairwise term
params['beta'] = 1  # weightening parameter for unary term
params['perc'] = 0.3  # what portion of liver parenchym around peak is used to calculate std of liver normal pdf
params['k_std_h'] = 3  # weighting parameter for sigma of normal distribution of healthy parenchym
params['k_std_t'] = 3  # weighting parameter for sigma of normal distribution of tumor
params['tv_weight'] = 0.05  # weighting parameter for total variation filter
params['healthy_simple_estim'] = False  # simple healthy parenchym pdf estimation from all data
params['prob_w'] = 0.5  # prob_w * max_prob is a threshold for data that will be used for estimation of other pdfs

params['hack_hypo_mu'] = -0  # hard move of mean of hypodense pdf to the left
params['hack_hypo_sigma'] = 0  # hard widening of sigma of hypodense pdf
params['hack_hyper_mu'] = 5  # hard move of mean of hyperdense pdf to the right
params['hack_hyper_sigma'] = 5  # hard widening of sigma of hyperdense  pdf

params['show_healthy_pdf_estim'] = False
params['show_estimated_pdfs'] = True
params['show_outlier_pdf_estim'] = False

params['hypo_label'] = 0  # label of hypodense objects
params['hyper_label'] = 2  # label of hyperdense objects
params['min_area'] = 20
params['min_compactness'] = 0.2

data_o = np.load('input_orig_data.npy')
res = np.load('res.npy')

hypo_lab = params['hypo_label']
hyper_lab = params['hyper_label']

# print 'calculating features of hypodense tumors...'
# labels_hypo, n_labels = scindimea.label(res == hypo_lab)
# labels_hypo -= 1
# areas_hypo = np.zeros(n_labels)
# comps_hypo = get_compactness(labels_hypo)
# for i in range(n_labels):
#     lab = labels_hypo == i
#     areas_hypo[i] = lab.sum()
#     print 'label = %i, area = %i, comp = %.3f' % (i, areas_hypo[i], comps_hypo[i])
#     # py3DSeedEditor.py3DSeedEditor(data_o, contour=lab).show()
# print '\t...done'

print 'calculating features of hyperdense tumors...'
labels_hyper, n_labels = scindimea.label(res == hyper_lab)
labels_hyper -= 1
areas_hyper = np.zeros(n_labels)
comps_hyper = get_compactness(labels_hyper)
for i in range(n_labels):
    lab = labels_hyper == i
    areas_hyper[i] = lab.sum()
    print 'label = %i, area = %i, comp = %.3f' % (i, areas_hyper[i], comps_hyper[i])
    # py3DSeedEditor.py3DSeedEditor(data_o, contour=lab).show()
print '\t...done'

print 'filtering false objects...'
features = ('area', 'compactness')
# features_hypo_v = np.vstack((areas_hypo, comps_hypo)).T
features_hyper_v = np.vstack((areas_hyper, comps_hyper)).T
# hypo_ok = filter_objects(features_hypo_v, features, params).sum(axis=1) == len(features)
hyper_ok = filter_objects(features_hyper_v, features, params).sum(axis=1) == len(features)

# print '\tfiltrated hypodense: %i/%i' % (hypo_ok.sum(), len(hypo_ok))
print '\tfiltrated hyperdense: %i/%i survived' % (hyper_ok.sum(), len(hyper_ok))


# hypo = np.in1d(labels_hypo, np.nonzero(hypo_ok)).reshape(labels_hypo.shape)
hyper = np.in1d(labels_hyper, np.nonzero(hyper_ok)).reshape(labels_hyper.shape)
py3DSeedEditor.py3DSeedEditor(data_o, contour=hyper).show()


__author__ = 'tomas'
import sys
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

from PyQt4.QtGui import QApplication, QFrame, QVBoxLayout
from PyQt4.QtCore import Qt

import numpy as np
import skimage.exposure as skiexp

from area_hist_widget_GUI import Ui_Frame

class AreaHistWidget(QFrame):

    def __init__(self, data=None):
        # Initialize the object as a QWidget
        QFrame.__init__(self)

        self.ui = Ui_Frame()
        self.ui.setupUi(self)
        # We have to set the size of the main window
        # ourselves, since we control the entire layout
        self.setMinimumSize(400, 250)
        self.setWindowTitle('Histogram of selected area')

        self.data = data

        if data is not None:
            self.set_data(data)

        self.init_UI_form()

    def set_data(self, data):
        if data is None:
            return

        self.data = data

        self.hist, self.bins = skiexp.histogram(self.data, nbins=256)
        self.update_figures()

        mean = self.data.mean()
        std = self.data.std()
        self.ui.mean_LBL.setText('%.1f' % mean)
        self.ui.std_LBL.setText('%.1f' % std)

    def init_UI_form(self):
        self.figure = plt.figure()
        # self.axes = self.figure.add_axes([0, 0, 1, 1])
        self.axes = self.figure.add_axes()

        self.canvas = FigureCanvas(self.figure)

        layout = QVBoxLayout()
        layout.addWidget(self.canvas, 1)
        self.ui.hist_F.setLayout(layout)

        # compute histogram
        if self.data is not None:
            self.hist, self.bins = skiexp.histogram(self.data, nbins=256)
            self.update_figures()

    def update_figures(self):
        plt.figure(self.figure.number)

        plt.plot(self.bins, self.hist, 'b')
        ax = plt.axis()
        plt.axis([ax[0]-5, ax[1]+5, ax[2], ax[3]])
        plt.hold(False)

        self.canvas.draw()

    # def keyPressEvent(self, QKeyEvent):
    #     print 'hist widget key event: ',
    #     if QKeyEvent.key() == Qt.Key_Escape:
    #         print 'Escape'
    #         self.close()


################################################################################
################################################################################
if __name__ == '__main__':
# runing application -------------------------
    app = QApplication(sys.argv)
    ahw = AreaHistWidget()
    data = np.array([1, 2, 3, 4, 5, 2, 3, 4, 3])
    ahw.set_data(data)
    ahw.show()
    sys.exit(app.exec_())


__author__ = 'tomas'

import sys
from PyQt4 import QtGui, QtCore
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
import numpy as np

# Main widget containing figures etc
class Form_widget(QtGui.QWidget):

    def __init__(self, window, cc):

        self.win = window  # link to the main window
        self.cc = cc  # link to the computational core
        # self.im = self.win.data  # input data
        # self.im = self.cc.data_1.data  # input data
        # self.labels = self.win.labels  # input labeling
        # self.labels = self.cc.data_1.labels  # input labeling
        # self.actual_slice_1 = self.win.view_1_curr_idx  # index of current data slice
        # self.actual_slice_2 = self.win.view_2_curr_idx  # index of current data slice
        # self.n_slices = self.im.shape[0]  # numer of slices
        self.healthy_label = self.win.params['healthy_label']
        self.hypo_label = self.win.params['hypo_label']
        self.hyper_label = self.win.params['hyper_label']
        self.win_w = self.win.params['win_width']
        self.win_l = self.win.params['win_level']

        super(Form_widget, self).__init__()
        self.init_UI_form()


    def init_UI_form(self):

        # QtGui.QToolTip.setFont(QtGui.QFont('SansSerif', 10))
        # self.data_l = self.im  # data to be shown in view_1
        # self.data_r = self.im  # data to be shown in view_2
        if self.win.ui.figure_1_CB.currentIndex() == 0:
            self.data_L = self.cc.data_1
        elif self.win.ui.figure_1_CB.currentIndex() == 1:
            self.data_L = self.cc.data_2
        if self.win.ui.figure_2_CB.currentIndex() == 0:
            self.data_R = self.cc.data_1
        elif self.win.ui.figure_1_CB.currentIndex() == 1:
            self.data_R = self.cc.data_2

        self.data_L_str = 'im'
        self.data_R_str = 'im'

        self.figure = plt.figure()
        self.axes = self.figure.add_axes([0, 0, 1, 1])

        self.canvas = FigureCanvas(self.figure)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.canvas, 1)
        self.setLayout(layout)

        # conenction to wheel events
        self.canvas.mpl_connect('scroll_event', self.on_scroll)

        self.update_figures()


    def label2rgb(self, slice):
        r = slice == self.hyper_label
        g = slice == self.healthy_label
        b = slice == self.hypo_label
        slice_rgb = np.dstack((r, g, b))

        return slice_rgb


    def window_slice(self, ctslice):
        if self.win_w > 0:
            mul = 255. / float(self.win_w)
        else:
            mul = 0

        lb =self.win_l - self.win_w / 2
        # aux = (ctslice.ravel(order='F') - lb) * mul
        aux = (ctslice - lb) * mul
        # idxs = np.where(aux < 0)[0]
        # aux[idxs] = 0
        # idxs = np.where(aux > 255)[0]
        # aux[idxs] = 255
        aux = np.where(aux < 0, 0, aux)
        aux = np.where(aux > 255, 255, aux)

        return aux.astype(np.uint8)


    def update_figures(self):
        # setting the minimal and maximal to scale luminance data
        if self.data_L_str is 'labels':
            vmin1 = self.data_L.labels.min()
            vmax1 = self.data_L.labels.max()
            slice_L = self.label2rgb(self.data_L.labels[self.win.view_L_curr_idx, :, :])
        else:
            vmin1 = 0
            vmax1 = 255
            # slice_L = self.data_L.labels[self.win.view_L_curr_idx, :, :]
            slice_L = self.window_slice(self.data_L.data[self.win.view_L_curr_idx, :, :])
        if self.data_R_str is 'labels':
            vmin2 = self.data_R.labels.min()
            vmax2 = self.data_R.labels.max()
            # try:
            #     slice = self.data_R.labels[self.win.view_2_curr_idx, :, :]
            #     heal = np.where(slice == self.healthy_label)
                # ok_labels = self.data_R.labels_v[self.cc.filtered_idxs]
                # slice_R = np.where(np.in1d(slice, ok_labels).reshape(slice.shape), slice, -1)
                # slice_R = np.where(heal, self.healthy_label, slice_R)
                # slice_R = self.label2rgb(slice_R)
            # except:
            #     slice_R = self.label2rgb(self.data_2[self.actual_slice, :, :])
            slice_R = self.label2rgb(self.data_R.labels[self.win.view_R_curr_idx, :, :])
        else:
            vmin2 = 0
            vmax2 = 255
            if self.win.disp_smoothed:
                vmax2 = self.labels.max()
            slice_R = self.window_slice(self.data_R.data[self.win.view_R_curr_idx, :, :])

        # preparing slice ---------------------------------------------------
        # if both views are enabled
        if self.win.show_view_1 and self.win.show_view_2:
            plt.figure(self.figure.number)
            plt.subplot(121)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_L, 'gray', interpolation='nearest', vmin=vmin1, vmax=vmax1)
            # displaying contours if desirable
            if self.data_L_str is 'contours':
                self.draw_contours(self.data_L.labels[self.win.view_L_curr_idx, :, :])
            plt.title('view_L: %s' % self.data_L_str)

            plt.subplot(122)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_R, 'gray', interpolation='nearest', vmin=vmin2, vmax=vmax2)
            # plt.imshow(slice_2, interpolation='nearest')
            # displaying contours if desirable
            if self.data_R_str is 'contours':
                self.draw_contours(self.data_R.labels[self.win.view_R_curr_idx, :, :])
            plt.title('view_R: %s' % self.data_R_str)

        # if only the first view is enabled
        elif self.win.show_view_1:
            plt.figure(self.figure.number)
            plt.subplot(111)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_L, 'gray', interpolation='nearest', vmin=vmin1, vmax=vmax1)
            if self.data_L_str is 'contours':
                self.draw_contours(self.data_L.labels[self.win.view_L_curr_idx, :, :])
            plt.title('view_L: %s' % self.data_L_str)

        # if only the second view is enabled
        elif self.win.show_view_2:
            plt.figure(self.figure.number)
            plt.subplot(111)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_R, 'gray', interpolation='nearest', vmin=vmin2, vmax=vmax2)
            if self.data_R_str is 'contours':
                self.draw_contours(self.data_R.labels[self.win.view_R_curr_idx, :, :])
            plt.title('view_R: %s' % self.data_R_str)
        else:
            plt.figure(self.figure.number)
            plt.clf()

        self.canvas.draw()


    def draw_contours(self, labels):
        plt.hold(True)
        try:
            self.figure.gca().contour(labels == self.healthy_label, [0.1], colors='g', linewidths=2)
            plt.hold(True)
        except:
            print 'contour fail: ', sys.exc_info()[0]
        try:
            # self.figure.gca().contour(self.labels[self.actual_slice, :, :] == self.hypo_label, [0.2], colors='b', linewidths=2)
            self.figure.gca().contour(labels == self.hypo_label, [0.1], colors='b', linewidths=2)
            plt.hold(True)
        except:
            print 'contour fail: ', sys.exc_info()[0]
        try:
            # self.figure.gca().contour(self.labels[self.actual_slice, :, :] == self.hyper_label, [0.2], colors='r', linewidths=2)
            self.figure.gca().contour(labels == self.hyper_label, [0.1], colors='r', linewidths=2)
            plt.hold(False)
        except:
            print 'contour fail: ', sys.exc_info()[0]

    # def update_slice_label(self):
    #     self.slice_label.setText('slice #: %i/%i' % (self.actual_slice + 1, self.n_slices))


    def next_slice(self):
        if self.win.view_L_curr_idx < self.data_L.n_slices - 1:
            self.win.view_L_curr_idx += 1
        if self.win.view_R_curr_idx < self.data_R.n_slices - 1:
            self.win.view_R_curr_idx += 1


    def prev_slice(self):
        if self.win.view_L_curr_idx > 0:
            self.win.view_L_curr_idx -= 1
        if self.win.view_R_curr_idx > 0:
            self.win.view_R_curr_idx -= 1


    def scroll_next(self):
        self.next_slice()
        self.update_figures()


    def scroll_prev(self):
        self.prev_slice()
        self.update_figures()


    def on_scroll(self, event):
        '''mouse wheel is used for setting slider value'''
        if event.button == 'up':
            self.next_slice()
        if event.button == 'down':
            self.prev_slice()
        # self.slider.setValue(self.actual_slice)
        # self.slider_change(self.actual_slice)
        self.update_figures()
        self.win.slice_change(self.win.view_L_curr_idx)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'simple_viewer_widget.ui'
#
# Created: Wed Jan 14 16:04:38 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName(_fromUtf8("Form"))
        Form.resize(499, 485)
        self.verticalLayout = QtGui.QVBoxLayout(Form)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.frame_2 = QtGui.QFrame(Form)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_2.setObjectName(_fromUtf8("frame_2"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.frame_2)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.viewer_F = QtGui.QFrame(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.viewer_F.sizePolicy().hasHeightForWidth())
        self.viewer_F.setSizePolicy(sizePolicy)
        self.viewer_F.setMinimumSize(QtCore.QSize(0, 0))
        self.viewer_F.setFrameShape(QtGui.QFrame.Box)
        self.viewer_F.setObjectName(_fromUtf8("viewer_F"))
        self.verticalLayout_2.addWidget(self.viewer_F)
        self.horizontalLayout_11 = QtGui.QHBoxLayout()
        self.horizontalLayout_11.setObjectName(_fromUtf8("horizontalLayout_11"))
        self.slice_number_LBL = QtGui.QLabel(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.slice_number_LBL.sizePolicy().hasHeightForWidth())
        self.slice_number_LBL.setSizePolicy(sizePolicy)
        self.slice_number_LBL.setMinimumSize(QtCore.QSize(100, 0))
        self.slice_number_LBL.setObjectName(_fromUtf8("slice_number_LBL"))
        self.horizontalLayout_11.addWidget(self.slice_number_LBL)
        self.slice_scrollB = QtGui.QScrollBar(self.frame_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.slice_scrollB.sizePolicy().hasHeightForWidth())
        self.slice_scrollB.setSizePolicy(sizePolicy)
        self.slice_scrollB.setOrientation(QtCore.Qt.Horizontal)
        self.slice_scrollB.setObjectName(_fromUtf8("slice_scrollB"))
        self.horizontalLayout_11.addWidget(self.slice_scrollB)
        self.verticalLayout_2.addLayout(self.horizontalLayout_11)
        self.verticalLayout.addWidget(self.frame_2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(_translate("Form", "Form", None))
        self.slice_number_LBL.setText(_translate("Form", "slice # =", None))




__author__ = 'tomas'

import numpy as np
import skimage.io as skiio
import skimage.exposure as skiexp
import sys

import matplotlib.pyplot as plt

from PyQt4.QtCore import Qt, QSize, QString, SIGNAL
from PyQt4.QtGui import QImage, QMainWindow,\
    QApplication, QSlider, QPushButton,\
    QLabel, QPixmap, QPainter, qRgba,\
    QComboBox, QIcon, QStatusBar,\
    QHBoxLayout, QVBoxLayout, QFrame, QSizePolicy


# BGRA order
GRAY_COLORTABLE = np.array([[ii, ii, ii, 255] for ii in range(256)], dtype=np.uint8)

class ImageViewer(QMainWindow):
    def __init__(self, img):
        super(ImageViewer, self).__init__()

        self.image = img

        # viewer frame
        self.frame_viewer = QFrame()
        self.view_L = SliceBox(self.image)
        self.view_L.setFrameShape(QFrame.Box)
        # self.view_L.setSlice(self.image)
        self.view_L.setMinimumSize(QSize(1,1))
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.view_L.setSizePolicy(sizePolicy)

        self.view_R = SliceBox(self.image)
        self.view_R.setFrameShape(QFrame.Box)
        # self.view_R.setSlice(self.image)
        self.view_R.setMinimumSize(QSize(1,1))
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.view_R.setSizePolicy(sizePolicy)
        self.view_R.setVisible(False)

        self.layout_viewer = QHBoxLayout()
        self.layout_viewer.addWidget(self.view_L)
        self.layout_viewer.addWidget(self.view_R)
        self.frame_viewer.setLayout(self.layout_viewer)

        #tlacitkovy frame
        self.frame_btn = QFrame()
        self.btn = QPushButton('test')
        self.layout_btn = QHBoxLayout()
        self.layout_btn.addWidget(self.btn)
        self.frame_btn.setLayout(self.layout_btn)

        #centralni frame obsahujici frame s viewery a frame s tlacitkem
        self.frame_c = QFrame()
        self.layout_v = QVBoxLayout()
        self.layout_v.addWidget(self.frame_viewer)
        self.layout_v.addWidget(self.frame_btn)

        self.frame_c.setLayout(self.layout_v)

        self.setCentralWidget(self.frame_c)

        self.two_views = False

        self.btn.clicked.connect(self.btn_callback)

    def btn_callback(self):
        self.two_views = not self.two_views
        if self.two_views:
            self.view_R.setVisible(True)
            new_w = self.frame_viewer.width() / 2
            new_h = self.frame_viewer.height() / 2
            self.view_L.resizeSlice(new_w, new_h)
            self.view_R.resizeSlice(new_w, new_h)
        else:
            self.view_R.setVisible(False)

    def resizeEvent(self, event):
        # TODO: oba labely at maji stejnou velikost
        pass
         # if self.two_views:
         #    new_w = self.frame_viewer.width() / 2
         #    new_h = self.frame_viewer.height() / 2
         #    # new_w1 = self.width() / 2
         #    # new_h1 = self.height() / 2
         #    self.view_L.resizeSlice(new_w, new_h)
         #    self.view_R.resizeSlice(new_w, new_h)


class SliceBox(QLabel):

    def __init__(self, img):

        QLabel.__init__(self)

        self.setSlice(img)

        self.qimage = QImage(self.image.repeat(4),
                     self.imageshape[1], self.imageshape[0],
                     QImage.Format_RGB32).scaled(self.imagesize, aspectRatioMode=Qt.KeepAspectRatio)
        self.setPixmap(QPixmap.fromImage(self.qimage))
        self.orig_pixmap = self.pixmap().copy()


    def resizeSlice(self, w, h):
        self.setPixmap(self.orig_pixmap.scaled(w, h, Qt.KeepAspectRatio, Qt.SmoothTransformation))


    def resizeEvent(self, event):
        self.resizeSlice(self.width(), self.height())


    def updateSlice(self):
        if self.ctslice_rgba is None:
            return

        img = self.ctslice_rgba.copy()

        self.qimage = QImage(img.flatten(),
                      self.imageshape[1], self.imageshape[0],
                      QImage.Format_ARGB32).scaled(self.imagesize, aspectRatioMode=Qt.KeepAspectRatio)
        self.setPixmap(QPixmap.fromImage(self.qimage))


    def getSliceRGBA(self, ctslice):
        aux = ctslice.ravel(order='F')
        return aux.astype(np.uint8)


    def setSlice(self, ctslice=None, seeds=None, contours=None):

        if ctslice is not None:
            # ctslice = np.transpose(ctslice)
            self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]

        self.image = ctslice
        self.imageshape = ctslice.shape

        self.imagesize = QSize(self.imageshape[0], self.imageshape[1])

        self.updateSlice()


################################################################################
################################################################################
if __name__ == '__main__':
    fname = '/home/tomas/Dropbox/images/puma.png'
    img = skiio.imread(fname, as_grey=True)
    img = skiexp.rescale_intensity(img, (0,1), (0,255)).astype(np.uint8)

    app = QApplication(sys.argv)
    win = ImageViewer(img)
    win.show()
    sys.exit(app.exec_())